# -*- coding: utf-8 -*-
from __future__ import print_function
import Manyo as mm
#import ana.Reduction.BaseCommands as BaseCom
import utsusemi.DNA.ana.Reduction.SliceBase as Slice
import utsusemi.DNA.ana.Reduction.PeakFitBase as PF
import utsusemi.DNA.ana.Reduction.DataFormatBase as DFB

#import ana.Reduction.ElasticScanBase as ES
#import vis.MPlot as mp
#import vis.M2Plot as mp2
import os
import time
import math

#########################################
#
#def getDataOfDNA(runNo=314,HwParam=" ",LambParam=" ",QParam=" ",TimeParam=" ",initFile="GetQEDataOfDNA.ini",outputData="QE",fileTag="_",search_dir=""):
#    """
#    Load EventData and convert to QE map
#    - GetHistogramHW(runNo,dHW,hw_min,hw_max,tof_offset,startTime,endTime)
#    - DoMask(DAT)
#    - ILambdaCorrDNA(dat,ec)
#    - SolidAngleCorrDNA(dat)
#    - CreateQEMap(dat,startQ,endQ,deltaQ)
#
#    @param runNo        (int) Run Number
#    @param HwParam      (string)
#    @param LambParam    (string)
#    @param QParam       (string)
#    @param TimeParam    (string)
#    @param initFile     (string)
#    @param fileTag      (string)   
#    @retval ECM      (ElementContainerMatrix)
#    """
#
#    return ES.GetDataOfDNA(runNo, HwParam, LambParam, QParam, TimeParam, initFile, outputData, fileTag,search_dir)
#
#############################################################################

def sliceQE(dat,lowerQ,upperQ):
    """
    最小値、最大値を入力して、その範囲に含まれるECを返す。
    @param dat       (ElementContaienrArray)
    @param lowerQ    (double) lowerQ
    @param upperQ    (double) upperQ
    @retval ec    (ElementContainer)
    """
    return Slice.SliceQE(dat,lowerQ,upperQ)

##########################################

def setQbin(dat,initialQ,finalQ,deltaQ):
    """
    最小値、最大値、刻み幅を入力して、Qビニングを行う。
    @param dat       (ElementContaienrArray)
    @param initialQ  (double)
    @param finalQ    (double)
    @param deltaQ    (double)
    """
    return Slice.SliceQEall(dat,initialQ,finalQ,deltaQ)

##########################################

def setQbinbyText(dat,Q_list_string):
    """
    カンマ区切りテキストのテーブルを新しいQの境界として、Qビニングを行う。
    @param dat    (ElementContaienrArray)
    @param Q_list_string         (string)
    """
    return Slice.SliceQEbyText(dat,Q_list_string)

##########################################
#
def setEbinForECA(eca,initial,final,delta):
    """
    最小値、最大値、刻み幅を入力してECAをEビニングを行う。
    @param  eca        (ElementContaienrArray)
    @param  initial    (double)
    @param  final      (double)
    @param  delta      (double)
    """
    return Slice.OutputECABinning(eca,initial,final,delta)

##########################################
#
def setEbinForEC(ec,initial,final,delta):
    """
    最小値、最大値、刻み幅を入力してECをEビニングを行う。
    @param  ec         (ElementContaienr)
    @param  initial    (double)
    @param  final      (double)
    @param  delta      (double)
    """
    return Slice.OutputECBinning(ec,initial,final,delta)

##########################################

def setEbin(ec,initial,final,delta):
    """
    最小値、最大値、刻み幅を入力してEビニングを行う。
    @param  ec         (ElementContainer/Array)
    @param  initial    (double)
    @param  final      (double)
    @param  delta      (double)
    """
    if type(ec) == mm.ElementContainer:
        return setEbinForEC(ec,initial,final,delta)
    if type(ec) == mm.ElementContainerArray:
        return setEbinForECA(ec,initial,final,delta)

##########################################
#
#def setEbinForECA(eca,initial,final):
#    """
#    @param  eca        (ElementContaienrArray)
#    @param  initial    (double)
#    @param  final      (double)
#    """
#    return Slice.OutputPartOfECA(eca,initial,final)
#
##########################################
#
#def setEbinForEC(ec,initial,final):
#    """
#    @param  ec         (ElementContaienr)
#    @param  initial    (double)
#    @param  final      (double)
#    """
#    return Slice.OutputPartOfEC(ec,initial,final)
#
##########################################
#
#def setEbin(ec,initial,final):
#    """
#    @param  ec         (ElementContainer/Array)
#    @param  initial    (double)
#    @param  final      (double)
#    """
#    if type(ec) == mm.ElementContainer:
#        return setEbinForEC(ec,initial,final)
#    if type(ec) == mm.ElementContainerArray:
#        return setEbinForECA(ec,initial,final)
#
##########################################

def getIntegralHWData(dat,emin,emax):
    """
    I(Q,E)を入力し、入力した範囲にE方向に積分したI(Q)の数値リストを返す。
    @param dat    (ElementContaienrArray)
    @param emiin   (float)
    @param emax    (float)
    @retval ret    (list)
    """
    if dat==None:
        null_list=[]
        return [null_list,null_list,null_list]

    if emin==emax:

        dat_X_min=[dat.Put(i).PutX()[0] for i in range(dat.PutTableSize())]
        dat_X_max=[dat.Put(i).PutX()[-1] for i in range(dat.PutTableSize())]

        emin = min(dat_X_min)
        emax = max(dat_X_max)

    ec = getIntegralHW(dat,emin,emax)

    resultECX=ec.PutX()
    resultECY=ec.PutY()
    resultECE=ec.PutE()

    resultDataX=[(resultECX[i]+resultECX[i+1])*0.5 for i in range(resultECX.size()-1)]
    resultDataY=[resultECY[i] for i in range(resultECY.size())]
    resultDataE=[resultECE[i] for i in range(resultECE.size())]

    resultlist=[resultDataX,resultDataY,resultDataE]

    return resultlist
##########################################

def getIntegralHW(dat,emin,emax):
    """
    I(Q,E)を入力し、入力した範囲にE方向に積分したI(Q)のECを返す。
    @param dat     (ElementContaienrArray)
    @param emin    (double)
    @param emax    (double)
    @retval EC     (ElementContainer)
    """
    num_of_bin = dat.PutSize()
    Xbin = []
    Ybin = []
    Ebin = []
    import utsusemi.DNA.ana.Reduction.PeakIntegral as PI
    xinitial=dat(0).PutHeader().PutDoubleVector("XRANGE")
    xbuf = xinitial[0]
    Xbin.append(xbuf)

    for i in range(num_of_bin):
       ec = dat(i)
       hh = ec.PutHeader()
       Xrange = hh.PutDoubleVector("XRANGE")
       dX=(Xrange[1]-Xrange[0])
       XCenter=dX*0.5+Xrange[0]

       sum_val,sum_val_e = PI.GetECPeakIntegral(ec,emin,emax)

       if (sum_val == sum_val):
           xbuf=xbuf + (XCenter-xbuf)*2
           Xbin.append(xbuf)
           Ybin.append(sum_val)
           Ebin.append(sum_val_e)

    EC = mm.ElementContainer()
    EC.Add("x",Xbin,"meV")
    EC.Add("y",Ybin,"")
    EC.Add("e",Ebin)
    EC.SetKeys("x","y","e")
    
    return EC

##########################################

def selectFittingQbin(dat,fitlist):
    """
    カンマ区切り文字列で指定したBoolにより、Fittingを行うかどうかを書き込む。
    @param dat        (ElementContaienrArray)
    @param fitlist    (string)
    @retval None
    """
    eca_size = dat.PutSize()

    if (fitlist == "t" or fitlist == "T"):
        fitlist = "t"
        for i in range(eca_size-1):
            fitlist += ",t"
    if (fitlist == "f" or fitlist == "F"):
        fitlist = "f"
        for i in range(eca_size-1):
            fitlist += ",f"
    fitlist_v=fitlist.split(",")

    if len(fitlist_v)!=eca_size:
        print("eca_size and fitlist is incorrect!!")
    if len(fitlist_v)==eca_size:
        for j in range(eca_size):
            hh = dat(j).PutHeader()
            if (hh.CheckKey("FIT")==0):
                hh.Add("FIT",fitlist_v[j])
            if (hh.CheckKey("FIT")==1):
                hh.OverWrite("FIT",fitlist_v[j])
            dat(j).InputHeader(hh)

##########################################

def CheckQbin(dat):
    """
    Fittingを行うかどうかのCheck。標準出力へ書き込む。
    @param dat    (ElementContaienrArray)
    @retval eca None
    """
    print("Qens >>  ## Check Fitting Q bin ##")
    print("Qens >>  Q_bin")
    print("Qens >>      // Fittinng")
    print("Qens >>  -------------------------")

    eca_size = dat.PutSize()
    for i in range(eca_size):
        Xrange = dat(i).PutHeaderPointer().PutDoubleVector("XRANGE")

        hh = dat(i).PutHeaderPointer()
        if (hh.CheckKey("FIT")==0):
            hh2 = "Not Setting"
        if (hh.CheckKey("FIT")==1):
            if (hh.PutString("FIT") == "t" or hh.PutString("FIT") == "T"):
                hh2 = "True"
            else:
                hh2 = "False"
        print("Qens >>  ",str(Xrange[0])) 
        print("Qens >>      > "+hh2)

    Xrange_final = dat(eca_size-1).PutHeaderPointer().PutDoubleVector("XRANGE")
    print("Qens >>  ",Xrange_final[1])
    Xbin = dat(0).PutX()
    emin = Xbin[0]
    emax = Xbin[Xbin.size()-1]

    return getIntegralHW(dat,emin,emax)

##########################################
def DoQEMask(eca, QEfilename):
    """
    QEマスクを実行する
    @param eca (ElementContainerArray)
    @param filename (string)
    @retval None
    """
    return Slice.DoQEMask(eca, QEfilename)

##########################################

def transpose(ccIn):
    """
    入力したListを転置させる。
    @param ccIn   (list)
    @retval ccOut (list)
    """
    ccOut = []
    for j in range(len(ccIn[0])):
        ccOut.append([ccIn[i][j] for i in range (len(ccIn))])

    return ccOut

##########################################

def getFittingParameterEC(result_eca,parameter,axis):
    """
    パラメータプロットの数値データを返す。
    @param result_eca  (ElementContainerArray)
    @param parameter   (int/string)
    @param axis        (string)
    @retval ecOut      (ElementContainer)
    """
    xbinc = []
    xbin = []
    ybin = []
    ebin = []
    ebin2 = []
    ybinAll = []
    ebinAll = []

    ecOut=mm.ElementContainer()
    if type(result_eca)== mm.ElementContainer:
        print("ECA fitting(-multi fitting) is not performed")
        print("input is ElementContainer")
    if type(result_eca)== mm.ElementContainerArray:
        ecaSize = result_eca.PutSize()
        for i in range(ecaSize):
            if type(parameter)==str:
                hh = result_eca(i).PutHeaderPointer()
                XRange=hh.PutDoubleVector("XRANGE")
                CenterVal = (XRange[0]+XRange[1])*0.5
                xbinc.append(CenterVal)

                valAll,valeAll=checkFittingParam(-1,hh)
                ybinAll.append(valAll)
                ebinAll.append(valeAll)

        xbinNew=[]
        if len(ybinAll)>0:
            import utsusemi.DNA.ana.Reduction.ParamPlotBase as eisf 
            eisfY=eisf.eisf()

            print("Function >> ",parameter)
            param    = transpose(ybinAll)
            paramErr = transpose(ebinAll)
#            ybin=eisfY.main(parameter,param)
            ybin,ebin=eisfY.mainWithError(parameter,param,paramErr)

            eisfX=eisf.eisf()
            xbinNew,ebin2=eisfX.mainWithError(axis,param,paramErr)

        if (xbinNew==[]):
            xbin.append((-xbinc[1]+3*xbinc[0])*0.5)
            for i in range(len(xbinc)):
                xbin.append(2*xbinc[i]-xbin[i])
        else:
            xbin.append((-xbinNew[1]+3*xbinNew[0])*0.5)
            for i in range(len(xbinNew)):
                xbin.append(2*xbinNew[i]-xbin[i])

        ec = mm.ElementContainer()
        ec.Add("x",xbin)
        ec.Add("y",ybin)
        ec.Add("e",ebin)

        ec.SetKeys("x","y","e")

        hh = ec.PutHeader()
        if type(parameter)==str:
            hh.Add("Param",parameter)
        if type(parameter)==int:
            hh.Add("Param",str(parameter))

    print("")
    print("  [ Plot Fitting Parameter ]")
    print("  Parameter = "+str(parameter))
    print("  Q\tval")
    print("")
    for i in range(len(ybin)):
        print((xbin[i]+xbin[i+1])*0.5, ybin[i],ebin[i])
    return ec

##########################################

def getFittingParameterData(result_eca,parameter,axis):
    """
    パラメータプロットのElementContainerを返す。
    @param result_eca  (ElementContainerArray)
    @param parameter   (int/string)
    @param axis        (string)
    @retval resultlist      (list)
    """
    ec = getFittingParameterEC(result_eca,parameter,axis)

    resultECX=ec.PutX()
    resultECY=ec.PutY()
    resultECE=ec.PutE()

    resultDataX=[(resultECX[i]+resultECX[i+1])*0.5 for i in range(resultECX.size()-1)]
    resultDataY=[resultECY[i] for i in range(resultECY.size())]
    resultDataE=[resultECE[i] for i in range(resultECE.size())]
    flag = False
    for res in resultDataE:
        if res!=0.0:
            flag=True
    if flag:
        while 1:
            if 0.0 in resultDataE:
                ind = resultDataE.index(0.0)
                resultDataX.pop(ind)
                resultDataY.pop(ind)
                resultDataE.pop(ind)
            else:
                break

    resultlist=[resultDataX,resultDataY,resultDataE]

    return resultlist

##########################################

def checkFittingParam(Comp,hh):
    """
    ヘッダーに含まれるFitting結果の指定したパラメータ結果の値と誤差をタプルで返す。
    @param Comp  (int)
    @param hh    (HeaderBase)
    """
    if type(Comp)==int:
        param = str(Comp)
        if Comp>=0:
            if (hh.CheckKey("FittingParam"+param)==0):
                print("Fitting parameter no."+param+" is not available")
                val = 0
                vale = 0
            else:
                val = hh.PutDouble("FittingParam"+param)
                vale = hh.PutDouble("FittingParamErr"+param)
        else:
            val=[]
            vale=[]
            param="0"
            while hh.CheckKey("FittingParam"+param)==1:
                val_tmp = hh.PutDouble("FittingParam"+param)
                vale_tmp = hh.PutDouble("FittingParamErr"+param)

                if val_tmp=="nan":
                    val_tmp=0
                if vale_tmp=="nan":
                    vale_tmp=0

                val.append(val_tmp)
                vale.append(vale_tmp)

                param=str(int(param)+1)

            XRange=hh.PutDoubleVector("XRANGE")   #20130124 add-line, start
            CenterVal = (XRange[0]+XRange[1])*0.5
            val.append(CenterVal)
            vale.append(0)                        #20130124 add-line, end
    else:
        val = 0
        vale = 0

    return val,vale

#########################################

def getFittingParamResult(result):
    """
    全フィッティングパラメータ結果の値、誤差をQ毎のListにして返す
    @param result          (ElementContainerArray)
    """
    paramList=[]
    if type(result) == mm.ElementContainerMatrix:
        fit_result_eca = result.Put(0)
        num_of_Q = fit_result_eca.PutSize()
        hh=fit_result_eca(0).PutHeaderPointer()

        for i in range(num_of_Q):
            hh=fit_result_eca(i).PutHeaderPointer()
            paramListEachRes = []
            paramListEachErr = []

            if (hh.CheckKey("ChiSq")==1):
                chisq = hh.PutDouble("ChiSq")
                flug = True
                if chisq==chisq:
                    pnum = 0
                    while flug:
                        if (hh.CheckKey("FittingParam"+str(pnum))==1):
                            paramListEachRes.append(hh.PutDouble("FittingParam"+str(pnum)))
                            paramListEachErr.append(hh.PutDouble("FittingParamErr"+str(pnum)))
                            pnum += 1
                        if (hh.CheckKey("FittingParam"+str(pnum))==0):
                            flug = False
                elif chisq!=chisq:
                    pnum = 0
                    while flug:
                        if (hh.CheckKey("FittingParam"+str(pnum))==1):
                            paramListEachRes.append(0)
                            paramListEachErr.append(0)
                            pnum += 1
                        if (hh.CheckKey("FittingParam"+str(pnum))==0):
                            flug = False

            paramListEach = [paramListEachRes,paramListEachErr]
            paramList.append(paramListEach)

    if type(result) == mm.ElementContainerArray:

        fit_result_ec = result.Put(0)
        hh=fit_result_ec.PutHeaderPointer()
        paramListRes = []
        paramListErr = []
        flug = True
        if (hh.CheckKey("ChiSq")==1):
            chisq = hh.PutDouble("ChiSq")
            if chisq==chisq:
                pnum = 0
                while flug:
                    if (hh.CheckKey("FittingParam"+str(pnum))==1):
                        paramListRes.append(hh.PutDouble("FittingParam"+str(pnum)))
                        paramListErr.append(hh.PutDouble("FittingParamErr"+str(pnum)))
                        pnum += 1
                    if (hh.CheckKey("FittingParam"+str(pnum))==0):
                        flug = False
            elif chisq!=chisq:
                pnum = 0
                while flug:
                    if (hh.CheckKey("FittingParam"+str(pnum))==1):
                        paramListRes.append(hh.PutDouble(0))
                        paramListErr.append(hh.PutDouble(0))
                        pnum += 1
                    if (hh.CheckKey("FittingParam"+str(pnum))==0):
                        flug = False
        paramList = [paramListRes,paramListErr]

    return paramList

#########################################

def peakConvFitEC(ecDet,ecRes,function,param,param_ub,param_lb,link_value,fit_region,useDomain,NumOfItteration,plotflug):
    """
    PeakFitting script by using Levmar(with Covolution),for EC
    @param ecDet           (ElementContainer)
    @param ecRes           (ElementContainer)
    @param function        (string)
    @param param           (string)
    @param param_lb        (string)
    @param param_ub        (string)
    @param link_value      (string)
    @param useDomain       (bool)
    @retval result_eca     (ElementContainerArray)
    """
    result_eca = mm.ElementContainerArray()
    function=replacePlusStr(function)
    result_ec_buf =  PF.PeakConvFitEC(ecDet,ecRes,function,param,param_ub,param_lb,link_value,fit_region,useDomain,NumOfItteration,plotflug)

    if (result_ec_buf.PutY())[0]!=(result_ec_buf.PutY())[0]:
        result_ec = mm.ElementContainer(result_ec_buf.PutHeader())
        result_ec.Add("x",(result_ec_buf.PutX()))
        result_ec.Add("y",[0]*(len(result_ec_buf.PutY())))
        result_ec.Add("e",[0]*(len(result_ec_buf.PutY())))
        result_ec.SetKeys("x","y","e")
    else:
        result_ec = result_ec_buf

    if type(param)==list:
        p1 = param[0]
    if type(param)!=list:
        p1 = param
    initial_ec = PF.getFittingFunction(function,p1,ecDet)
    residual_ec = PF.getResidualDistribution(ecDet,result_ec)

    result_ec.SetUnit(result_ec.PutXKey(),'meV')
    result_ec.SetUnit(result_ec.PutYKey(),'')
    initial_ec.SetUnit(initial_ec.PutXKey(),'meV')
    initial_ec.SetUnit(initial_ec.PutYKey(),'')
    residual_ec.SetUnit(residual_ec.PutXKey(),'meV')
    residual_ec.SetUnit(residual_ec.PutYKey(),'')
    ecDet.SetUnit(ecDet.PutXKey(),'meV')
    ecDet.SetUnit(ecDet.PutYKey(),'')

    result_eca.Add(result_ec)
    result_eca.Add(ecDet)
    result_eca.Add(initial_ec)
    result_eca.Add(residual_ec)
    result_eca.Add(ecRes)
    #ShowResultECFit(result_eca)
    return result_eca

##########################################

def peakConvFitECA(ecDet,ecRes,function,param,param_ub,param_lb,link_value,fit_region,useDomain,NumOfItteration,plotflug):
    """
    PeakFitting script by using Levmar,(with Covolution),for ECA
    @param ecDet           (ElementContainer)
    @param ecRes           (ElementContainer)
    @param function        (string)
    @param param           (string)
    @param param_lb        (string)
    @param param_ub        (string)
    @param link_value      (string)
    @param useDomain       (bool)
    @retval result_ecm     (ElementContainerMatrix)
    """
    function=replacePlusStr(function)
    result_eca_buf = PF.PeakConvFitECA(ecDet,ecRes,function,param,param_ub,param_lb,link_value,fit_region,useDomain,NumOfItteration,plotflug)

    initial_eca  = mm.ElementContainerArray()
    observed_eca = mm.ElementContainerArray()
    residual_eca = mm.ElementContainerArray()
    result_eca   = mm.ElementContainerArray()
    resolution_eca = mm.ElementContainerArray()
    result_ecm = mm.ElementContainerMatrix()

    for i in range(result_eca_buf.PutSize()):
        if ((result_eca_buf.Put(i)).PutY())[0]!=((result_eca_buf.Put(i)).PutY())[0]:

            result_tmp = mm.ElementContainer(result_eca_buf.Put(i).PutHeader())
            result_tmp.Add("x",(result_eca_buf.Put(i)).PutX())
            result_tmp.Add("y",[0]*(len((result_eca_buf.Put(i)).PutY())))
            result_tmp.Add("e",[0]*(len((result_eca_buf.Put(i)).PutY())))
            result_tmp.SetKeys("x","y","e")
        else:
            result_tmp = result_eca_buf.Put(i)

        result_tmp.SetUnit(result_tmp.PutXKey(),'meV')
        result_tmp.SetUnit(result_tmp.PutYKey(),'')
        result_eca.Add(result_tmp)

    j=0
    for i in range(ecDet.PutSize()):
        PF.SetResolutionData(ecRes(i))
        hh = ecDet(i).PutHeaderPointer()
        fit_flug = False
        if (hh.CheckKey("FIT")==1):
            if (hh.PutString("FIT") == "t" or hh.PutString("FIT") == "T"):
                fit_flug = True
        elif (hh.CheckKey("FIT")==0):
                hh.Add("FIT","t")
                fit_flug = True
        if fit_flug:
            if type(param) == list:
                if i<len(param):
                    initial_ec = PF.getFittingFunction(function,param[i],ecDet(i))
                else:
                    initial_ec = PF.getFittingFunction(function,param[len(param)-1],ecDet(i))
            if type(param) == str:
                initial_ec = PF.getFittingFunction(function,param,ecDet(i))
            if (type(param) == str) or (type(param) == list):
                initial_ec.SetUnit(initial_ec.PutXKey(),'meV')
                initial_ec.SetUnit(initial_ec.PutYKey(),'')
                initial_eca.Add(initial_ec)
            residual_ec = PF.getResidualDistribution(ecDet(i),result_eca(j))
            residual_ec.SetUnit(initial_ec.PutXKey(),'meV')
            residual_ec.SetUnit(initial_ec.PutYKey(),'')
            ecDet(i).SetUnit(ecDet(i).PutXKey(),'meV')
            ecDet(i).SetUnit(ecDet(i).PutYKey(),'')
            residual_eca.Add(residual_ec)
            observed_eca.Add(ecDet(i))
            resolution_eca.Add(ecRes(i))
            j+=1
        else:#20121226
            if type(param) == list:
                if i<len(param):
                    initial_ec = PF.getFittingFunction(function,param[i],ecDet(i))
                else:
                    initial_ec = PF.getFittingFunction(function,param[len(param)-1],ecDet(i))
            if type(param) == str:
                initial_ec = PF.getFittingFunction(function,param,ecDet(i))
            if (type(param) == str) or (type(param) == list):
                initial_ec.SetUnit(initial_ec.PutXKey(),'meV')
                initial_ec.SetUnit(initial_ec.PutYKey(),'')
                initial_eca.Add(initial_ec)
            residual_ec = PF.getResidualDistribution(ecDet(i),initial_ec)
            residual_ec.SetUnit(initial_ec.PutXKey(),'meV')
            residual_ec.SetUnit(initial_ec.PutYKey(),'')
            ecDet(i).SetUnit(ecDet(i).PutXKey(),'meV')
            ecDet(i).SetUnit(ecDet(i).PutYKey(),'')
            residual_eca.Add(residual_ec)
            observed_eca.Add(ecDet(i))
            resolution_eca.Add(ecRes(i))
            j+=1

    result_ecm.Add(result_eca)
    result_ecm.Add(observed_eca)
    result_ecm.Add(initial_eca)
    result_ecm.Add(residual_eca)
    result_ecm.Add(resolution_eca)

    # re-define result_eca and rsidual distribution
    # Manyoに組み込んだ関数の結果出力のビン数が一つ少ないので計算をし直す。
     # それに伴い残渣も計算し直す。(test)


    result_eca=getTotalResultDistribution(result_ecm)
    while result_ecm.PutTableSize()!=0:
        result_ecm.EraseElement(0)

    result_ecm.Add(result_eca)
    result_ecm.Add(observed_eca)
    result_ecm.Add(initial_eca)
    result_ecm.Add(residual_eca)
    result_ecm.Add(resolution_eca)

    residual_eca=getResidualDistribution(result_ecm)
    result_ecm.EraseElement(4)
    result_ecm.EraseElement(3)
    result_ecm.Add(residual_eca)
    result_ecm.Add(resolution_eca)

    return result_ecm

##########################################

def peakConvFit(ecDet,ecRes,function,param,param_ub,param_lb,link_value,fit_region,useDomain,NumOfItteration,plotflug):
    """
    PeakFitting script by using Levmar, (with Covolution)
    @param ecDet           (ElementContainer)
    @param ecRes           (ElementContainer)
    @param function        (string)
    @param param           (string)
    @param param_lb        (string)
    @param param_ub        (string)
    @param link_value      (string)
    @param useDomain       (bool)
    @retval result_ecm     (ElementContainerMatrix)
    """
    if type(ecDet) == mm.ElementContainer:
        return peakConvFitEC(ecDet,ecRes,function,param,param_ub,param_lb,link_value,fit_region,useDomain,NumOfItteration,plotflug)
    if type(ecDet) == mm.ElementContainerArray:
        return peakConvFitECA(ecDet,ecRes,function,param,param_ub,param_lb,link_value,fit_region,useDomain,NumOfItteration,plotflug)

##########################################

def peakFitEC(ecDet,function,param,param_ub,param_lb,link_value,fit_region,useDomain,NumOfItteration,plotflug):
    """
    PeakFitting script by using Levmar, (without Covolution),for EC
    @param ecDet           (ElementContainer)
    @param function        (string)
    @param param           (string)
    @param param_lb        (string)
    @param param_ub        (string)
    @param link_value      (string)
    @param useDomain       (bool)
    @retval result_eca     (ElementContainerArray)
    """
    result_eca = mm.ElementContainerArray()
    function=replacePlusStr(function)
    result_ec_buf = PF.PeakFitEC(ecDet,function,param,param_ub,param_lb,link_value,fit_region,useDomain,NumOfItteration,plotflug)

    if (result_ec_buf.PutY())[0]!=(result_ec_buf.PutY())[0]:
        result_ec = mm.ElementContainer(result_ec_buf.PutHeader())
        result_ec.Add("x",(result_ec_buf.PutX()))
        result_ec.Add("y",[0]*(len(result_ec_buf.PutY())))
        result_ec.Add("e",[0]*(len(result_ec_buf.PutY())))
        result_ec.SetKeys("x","y","e")
    else:
        result_ec = result_ec_buf

    initial_ec = PF.getFittingFunction(function,param,ecDet)
    residual_ec = PF.getResidualDistribution(ecDet,result_ec)

    result_eca.Add(result_ec)
    result_eca.Add(ecDet)
    result_eca.Add(initial_ec)
    result_eca.Add(residual_ec)
    result_eca.Add(ecRes)

    #ShowResultECFit(result_eca)
    return result_eca

##########################################

def peakFitECA(ecDet,function,param,param_ub,param_lb,link_value,fit_region,useDomain,NumOfItteration,plotflug):
    """
    PeakFitting script by using Levmar, (without Covolution),for ECA
    @param ecDet           (ElementContainer)
    @param function        (string)
    @param param           (string)
    @param param_lb        (string)
    @param param_ub        (string)
    @param link_value      (string)
    @param useDomain       (bool)
    @retval result_ecm     (ElementContainerMatrix)
    """
    initial_eca  = mm.ElementContainerArray()
    observed_eca = mm.ElementContainerArray()
    resolution_eca = mm.ElementContainerArray()
    residual_eca = mm.ElementContainerArray()
    result_eca   = mm.ElementContainerArray()

    result_ecm = mm.ElementContainerMatrix()
    function=replacePlusStr(function)
    result_eca_buf = PF.PeakFitECA(ecDet,function,param,param_ub,param_lb,link_value,fit_region,useDomain,NumOfItteration,plotflug)

    for i in range(result_eca_buf.PutSize()):
        if ((result_eca_buf.Put(i)).PutY())[0]!=((result_eca_buf.Put(i)).PutY())[0]:

            result_tmp = mm.ElementContainer(result_eca_buf.Put(i).PutHeader())
            result_tmp.Add("x",(result_eca_buf.Put(i)).PutX())
            result_tmp.Add("y",[0]*(len((result_eca_buf.Put(i)).PutY())))
            result_tmp.Add("e",[0]*(len((result_eca_buf.Put(i)).PutY())))
            result_tmp.SetKeys("x","y","e")
        else:
            result_tmp = result_eca_buf.Put(i)

        result_tmp.SetUnit(result_tmp.PutXKey(),'meV')
        result_tmp.SetUnit(result_tmp.PutYKey(),'')
        result_eca.Add(result_tmp)

    j=0
    for i in range(ecDet.PutSize()):
        hh = ecDet(i).PutHeaderPointer()
        fit_flug = False
        if (hh.CheckKey("FIT")==1):
            if (hh.PutString("FIT") == "t" or hh.PutString("FIT") == "T"):
                fit_flug = True
        elif (hh.CheckKey("FIT")==0):
                hh.Add("FIT","t")
                fit_flug = True
        if fit_flug:
            if type(param) == list:
                if i<len(param):
                    initial_ec = PF.getFittingFunction(function,param(i),ecDet(i))
                else:
                    initial_ec = PF.getFittingFunction(function,param(len(param)-1),ecDet(i))
            if type(param) == str:
                initial_ec = PF.getFittingFunction(function,param,ecDet(i))
            if (type(param) == str) or (type(param) == list):
                initial_ec.SetUnit(initial_ec.PutXKey(),'meV')
                initial_ec.SetUnit(initial_ec.PutYKey(),'')
                initial_eca.Add(initial_ec)
            residual_ec = PF.getResidualDistribution(ecDet(i),result_eca(j))
            residual_ec.SetUnit(initial_ec.PutXKey(),'meV')
            residual_ec.SetUnit(initial_ec.PutYKey(),'')
            ecDet(i).SetUnit(ecDet(i).PutXKey(),'meV')
            ecDet(i).SetUnit(ecDet(i).PutYKey(),'')
            residual_eca.Add(residual_ec)
            observed_eca.Add(ecDet(i))
            resolution_eca.Add(ecRes(i))
            j+=1

    result_ecm.Add(result_eca)
    result_ecm.Add(observed_eca)
    result_ecm.Add(initial_eca)
    result_ecm.Add(residual_eca)
    result_ecm.Add(resolution_eca)
    #ShowResultECAFit(result_ecm)
    return result_ecm

##########################################

def peakFit(ecDet,function,param,param_ub,param_lb,link_value,fit_region,useDomain,NumOfItteration,plotflug):
    """
    PeakFitting script by using Levmar, (without Covolution)
    @param ecDet           (ElementContainer)
    @param ecRes           (ElementContainer)
    @param function        (string)
    @param param           (string)
    @param param_lb        (string)
    @param param_ub        (string)
    @param link_value      (string)
    @param useDomain       (bool)
    @retval result_ecm     (ElementContainerMatrix)
    """
    if type(ecDet) == mm.ElementContainer:
        return peakFitEC(ecDet,function,param,param_ub,param_lb,link_value,fit_region,useDomain,NumOfItteration,plotflug)
    if type(ecDet) == mm.ElementContainerArray:
        return peakFitECA(ecDet,function,param,param_ub,param_lb,link_value,fit_region,useDomain,NumOfItteration,plotflug)

##########################################

def getConvFitGuessEC(ecDet,ecRes,function,param,fit_region,plotflug):
    """
    Fitting初期値の計算、QensFitAnaでは未使用、(with Covolution) for EC
    @param ecDet           (ElementContainer)
    @param ecRes           (ElementContainer)
    @param function        (string)
    @param param           (string)
    @param param_lb        (string)
    @param param_ub        (string)
    @param useDomain       (bool)
    @retval result_eca     (ElementContainerArray)
    """
    result_eca = mm.ElementContainerArray()
    result_ec =  NoneEC()
    function=replacePlusStr(function)
    PF.SetResolutionData(ecRes)

    if type(param)==list:
        paramInit = param[0]
        print("input type of Param was list")
        print("first parameter was selected")
    if type(param)==str:
        paramInit = param

    initial_ec = PF.getFittingFunction(function,paramInit,ecDet)
    residual_ec = PF.getResidualDistribution(ecDet,initial_ec)

    result_ec.SetUnit(result_ec.PutXKey(),'meV')
    result_ec.SetUnit(result_ec.PutYKey(),'')
    initial_ec.SetUnit(initial_ec.PutXKey(),'meV')
    initial_ec.SetUnit(initial_ec.PutYKey(),'')
    residual_ec.SetUnit(residual_ec.PutXKey(),'meV')
    residual_ec.SetUnit(residual_ec.PutYKey(),'')
    ecDet.SetUnit(ecDet.PutXKey(),'meV')
    ecDet.SetUnit(ecDet.PutYKey(),'')

    result_eca.Add(result_ec)
    result_eca.Add(ecDet)
    result_eca.Add(initial_ec)
    result_eca.Add(residual_ec)
    result_eca.Add(ecRes)

    #ShowResultECFit(result_eca)
    return result_eca

##########################################

def getConvFitGuessECA(ecDet,ecRes,function,param,fit_region,plotflug):
    """
    Fitting初期値の計算、QensFitAnaでは未使用、(with Covolution) for ECA
    @param ecDet           (ElementContainer)
    @param ecRes           (ElementContainer)
    @param function        (string)
    @param param           (string)
    @retval result_ecm     (ElementContainerMatrix)
    """
    initial_eca  = mm.ElementContainerArray()
    observed_eca = mm.ElementContainerArray()
    residual_eca = mm.ElementContainerArray()
    result_eca = mm.ElementContainerArray()
    resolution_eca = mm.ElementContainerArray()
    result_ec   = NoneEC()
    function=replacePlusStr(function)
    result_ecm = mm.ElementContainerMatrix()
    for i in range(ecDet.PutSize()):
        hh = ecDet(i).PutHeaderPointer()
        PF.SetResolutionData(ecRes(i))
        fit_flug = False
        if (hh.CheckKey("FIT")==1):
            if (hh.PutString("FIT") == "t" or hh.PutString("FIT") == "T"):
                fit_flug = True
        elif (hh.CheckKey("FIT")==0):
                hh.Add("FIT","t")
                fit_flug = True
        #fit_flug = True
        if fit_flug:
            if type(param) == list:
                if i<len(param):
                    initial_ec = PF.getFittingFunction(function,param[i],ecDet(i))
                else:
                    initial_ec = PF.getFittingFunction(function,param[len(param)-1],ecDet(i))
            if type(param) == str:
                initial_ec = PF.getFittingFunction(function,param,ecDet(i))
            if (type(param) == str) or (type(param) == list):
                initial_ec.SetUnit(initial_ec.PutXKey(),'meV')
                initial_ec.SetUnit(initial_ec.PutYKey(),'')
                initial_eca.Add(initial_ec)
            residual_ec = PF.getResidualDistribution(ecDet(i),initial_ec)
            residual_ec.SetUnit(initial_ec.PutXKey(),'meV')
            residual_ec.SetUnit(initial_ec.PutYKey(),'')
            ecDet(i).SetUnit(ecDet(i).PutXKey(),'meV')
            ecDet(i).SetUnit(ecDet(i).PutYKey(),'')

            hh = ecDet(i).PutHeaderPointer()
            paramVal = param[i].split(",")
            for j in range(len(paramVal)):
                if (hh.CheckKey("FittingParam"+str(j))==0):
                    hh.Add("FittingParam"+str(j),float(paramVal[j]))
                if (hh.CheckKey("FittingParam"+str(j))==1):
                    hh.OverWrite("FittingParam"+str(j),float(paramVal[j]))
            for j in range(len(paramVal)):
                if (hh.CheckKey("FittingParamErr"+str(j))==0):
                    hh.Add("FittingParamErr"+str(j),0.0)
                if (hh.CheckKey("FittingParamErr"+str(j))==1):
                    hh.OverWrite("FittingParamErr"+str(j),0.0)
            if (hh.CheckKey("ChiSq")==0):
                hh.Add("ChiSq",0)
            if (hh.CheckKey("ChiSq")==1):
                hh.OverWrite("ChiSq",0)
            if (hh.CheckKey("FittingFunc")==0):
                hh.Add("FittingFunc",function)
            if (hh.CheckKey("FittingFunc")==1):
                hh.OverWrite("FittingFunc",function)
            residual_eca.Add(residual_ec)
            observed_eca.Add(ecDet(i))
            resolution_eca.Add(ecRes(i))
            result_eca.Add(result_ec)

    result_ecm.Add(result_eca)
    result_ecm.Add(observed_eca)
    result_ecm.Add(initial_eca)
    result_ecm.Add(residual_eca)
    result_ecm.Add(resolution_eca)

    return result_ecm

##########################################

def getConvFitGuess(ecDet,ecRes,function,param,fit_region,plotflug):
    """
    Fitting初期値の計算、QensFitAnaでは未使用、(with Covolution)
    @param ecDet           (ElementContainer/Array)
    @param ecRes           (ElementContainer/Array)
    @param function        (string)
    @param param           (string)
    @retval result_ecm     (ElementContainerArray/Matrix)
    """
    if type(ecDet) == mm.ElementContainer:
        return getConvFitGuessEC(ecDet,ecRes,function,param,fit_region,plotflug)
    if type(ecDet) == mm.ElementContainerArray:
        return getConvFitGuessECA(ecDet,ecRes,function,param,fit_region,plotflug)

##########################################

def getFitGuessEC(ecDet,function,param,fit_region,plotflug):
    """
    Fitting初期値の計算、QensFitAnaでは未使用、(without Covolution) for ECA
    @param ecDet           (ElementContainer)
    @param function        (string)
    @param param           (string)
    @retval result_eca     (ElementContainerArray)
    """
    result_eca = mm.ElementContainerArray()
    function=replacePlusStr(function)
    result_ec = NoneEC()

    if type(param)==list:
        paramInit = param[0]
        print("input type of Param was list")
        print("first parameter was selected")
    if type(param)==str:
        paramInit = param

    initial_ec = PF.getFittingFunction(function,paramInit,ecDet)
    residual_ec = PF.getResidualDistribution(ecDet,initial_ec)

    result_eca.Add(result_ec)
    result_eca.Add(ecDet)
    result_eca.Add(initial_ec)
    result_eca.Add(residual_ec)

    #ShowResultECFit(result_eca)
    return result_eca

##########################################

def getFitGuessECA(ecDet,function,param,fit_region,plotflug):
    """
    Fitting初期値の計算、QensFitAnaでは未使用、(without Covolution) for ECA
    @param ecDet           (ElementContainerArray)
    @param function        (string)
    @param param           (string)
    @retval result_ecm     (ElementContainerMatrix)
    """

    initial_eca  = mm.ElementContainerArray()
    observed_eca = mm.ElementContainerArray()
    residual_eca = mm.ElementContainerArray()
    result_eca   = mm.ElementContainerArray()
    result_ec   = NoneEC()
    function=replacePlusStr(function)
    result_ecm = mm.ElementContainerMatrix()

    for i in range(ecDet.PutSize()):
        hh = ecDet(i).PutHeaderPointer()
        fit_flug = True
        if fit_flug:
            if type(param) == list:
                if i<len(param):
                    initial_ec = PF.getFittingFunction(function,param[i],ecDet(i))
                else:
                    initial_ec = PF.getFittingFunction(function,param(len(param)-1),ecDet(i))
            if type(param) == str:
                initial_ec = PF.getFittingFunction(function,param,ecDet(i))
            if (type(param) == str) or (type(param) == list):
                initial_ec.SetUnit(initial_ec.PutXKey(),'meV')
                initial_ec.SetUnit(initial_ec.PutYKey(),'')
                initial_eca.Add(initial_ec)
            residual_ec = PF.getResidualDistribution(ecDet(i),initial_ec)
            residual_ec.SetUnit(initial_ec.PutXKey(),'meV')
            residual_ec.SetUnit(initial_ec.PutYKey(),'')
            ecDet(i).SetUnit(ecDet(i).PutXKey(),'meV')
            ecDet(i).SetUnit(ecDet(i).PutYKey(),'')
            residual_eca.Add(residual_ec)
            observed_eca.Add(ecDet(i))
            result_eca.Add(result_ec)

    result_ecm.Add(result_eca)
    result_ecm.Add(observed_eca)
    result_ecm.Add(initial_eca)
    result_ecm.Add(residual_eca)

    #ShowResultECAFit(result_ecm)
    return result_ecm

##########################################

def getFitGuess(ecDet,function,param,fit_region,plotflug):
    """
    Fitting初期値の計算、QensFitAnaでは未使用、(without Covolution)
    @param ecDet           (ElementContainer/Array)
    @param function        (string)
    @param param           (string)
    @retval result_ecm     (ElementContainerArray/Matrix)
    """
    if type(ecDet) == mm.ElementContainer:
        return getFitGuessEC(ecDet,function,param,fit_region,plotflug)
    if type(ecDet) == mm.ElementContainerArray:
        return getFitGuessECA(ecDet,function,param,fit_region,plotflug)

##########################################

def NoneEC():
    """
    強度ZeroのECを返す
    @retval ec     (ElementContainer)
    """
    xbin = []
    ybin = []
    ebin = []

    for i in range(100):
        xbin.append(i)
        ybin.append(0)
        ebin.append(0)
    xbin.append(100)
    ec = mm.ElementContainer()
    ec.Add("x",xbin)
    ec.Add("y",ybin)
    ec.Add("e",ebin)
    ec.SetKeys("x","y","e")

    return ec

##########################################

def getFitEachResult(result):
    """
    Fitting結果の各関数要素をElementContainerMatrix/Arrayで返す
    @param result          (ElementContainerMatrix/Array)
    """
    if type(result)==mm.ElementContainerMatrix:
        result_eca = result.Put(1)
        ecaRes = result.Put(4)

        each_comp_ecm = mm.ElementContainerMatrix()

        NumECA = result_eca.PutSize()
        for i in range(NumECA):
            result_ec = result_eca(i)
            hh=result_ec.PutHeaderPointer()

            #if "xOrg" in result_ec.PutKeysList():
            #    result_ec.SetKeys("xOrg","yOrg","eOrg")

            ecRes = ecaRes(i)
            if (hh.CheckKey("FittingFunc")==1):
                func=hh.PutString("FittingFunc")
            if (hh.CheckKey("FittingFunc")==0):
                print("Fitting function is unknown")
            flug = True
            pnum = 0
            param = []
            while flug:
                if (hh.CheckKey("FittingParam"+str(pnum))==1):
                    param.append(hh.PutDouble("FittingParam"+str(pnum)))
                    #param.append(float(hh.PutDouble("FittingParam"+str(pnum))))
                    pnum += 1
                if (hh.CheckKey("FittingParam"+str(pnum))==0):
                    flug = False
            PF.SetResolutionData(ecRes)
            each_comp_eca=PF.getFittingEachFunction(func,param,result_ec)
            each_comp_ecm.Add(each_comp_eca)

        return each_comp_ecm

    if type(result)==mm.ElementContainerArray:
        result_ec = result.Put(1)
        hh=result_ec.PutHeaderPointer()
        if "xOrg" in result_ec.PutKeysList()==1:
            result_ec.SetKeys("xOrg","yOrg","eOrg")

        ecRes = result.Put(4)

        if (hh.CheckKey("FittingFunc")==1):
            func=hh.PutString("FittingFunc")
        if (hh.CheckKey("FittingFunc")==0):
            print("Fitting function is unknown")
        flug = True
        pnum = 0
        param = []
        while flug:
            if (hh.CheckKey("FittingParam"+str(pnum))==1):
                param.append(hh.PutDouble("FittingParam"+str(pnum)))
                pnum += 1
            if (hh.CheckKey("FittingParam"+str(pnum))==0):
                flug = False
        PF.SetResolutionData(ecRes)
        each_comp_eca=PF.getFittingEachFunction(func,param,result_ec)

        return each_comp_eca

##########################################

def getSigma(result):
    """
    FittingのSigmaを返す。
    @param result          (ElementContainerArray/Matrix)
    """
    if type(result) == mm.ElementContainerArray:

        residual = result.Put(3).PutY()
        error = result.Put(1).PutE()

        sumResidual = 0
        for i in range(residual.size()):
            if(error[i]!=0):
                sumResidual += residual[i]*residual[i]/error[i]/error[i]
        normResidual = sumResidual/residual.size()
        return normResidual

    if type(result) == mm.ElementContainerMatrix:

        residual_eca = result.Put(3)
        error_eca = result.Put(1)

        numEca = residual_eca.PutSize()

        normResidualList = []

        for i in range(numEca):
            residual = residual_eca.Put(i).PutY()
            error_ec=error_eca.Put(i)
            #if "xOrg" in error_ec.PutKeysList():       #20130305 start
            #    error_ec.SetKeys("xOrg","yOrg","eOrg") #20130305 end
            error = error_ec.PutE()
            #print "#",residual.size(),error.size()
            sumResidual = 0
            for i in range(residual.size()):
                if i <len(error):
                    if(error[i]!=0):
                        sumResidual += residual[i]*residual[i]/error[i]/error[i]

            if residual.size()!=0:
                normResidual = sumResidual/residual.size()
            else:
                normResidual = 0
                print("residual-list size is zero")
            normResidualList.append(normResidual)
        #print normResidualList
        return normResidualList

##########################################
#
#def showFitResultECA(result):
#    """
#    @param result          (ElementContainerMatrix)
#    """
#    p=mp.MPlot()
#    p.SetMainTitle("Fitting result")
#    p.SetYLabel("S(Q,E)")
#    p.SetXLabel("Hw")
#    eachComp = GetFitEachResult(result)
#    numComp = eachComp(0).PutSize()
# #    p.SetTraceNum(2)
# #    for i in range(result(0).PutSize()):
# #        p.AddData(result.Put(1).Put(i))
# #        p.AddData(result.Put(0).Put(i))
# #        p.SetYScale()
# #        p.SetPlotParam(i*2+1,mc="b",lc="b")
# #        p.SetPlotParam(i*2+2,ls="-",lw=8,mk="",eb=False,lc="r") 
# #        p.ChangePage(i+2)
#
#    p.SetTraceNum(2+numComp)
#    for i in range(result(0).PutSize()):
#        p.AddData(result.Put(1).Put(i))
#        p.AddData(result.Put(0).Put(i))
#        for j in range(numComp):
#            p.AddData(eachComp.Put(i).Put(j))
#        p.SetYScale()
#        p.SetPlotParam(i*(2+numComp)+1,mc="b",lc="b")
#        p.SetPlotParam(i*(2+numComp)+2,ls="-",lw=8,mk="",eb=False,lc="r")
#        for j in range(numComp):
#            p.SetPlotParam(i*(2+numComp)+3+j,ls="--",lw=8,mk="",eb=False,lc="g")
#        p.ChangePage(i+2+numComp)
#
#    p2=mp.MPlot()
#    p2.SetTraceNum(1)
#    p2.SetMainTitle("Residual Distribution")
#    p2.SetYLabel("(S(Q,E)$_{Obs}$ - S(Q,E)$_{Cal}$) / $\sigma$")
#    p2.SetXLabel("Hw")
#    #print (result.Put(3)).PutSize()
#    for i in range(result(0).PutSize()):
#        p2.AddData(result.Put(3).Put(i))
#        p2.SetPlotParam(i+1,mc="b",lc="b")
#        p2.ChangePage(i+1)
#
##########################################
#
#def showFitResultEC(result):
#    """
#    @param result          (ElementContainerArray)
#    """
#    eachComp = GetFitEachResult(result)
#    numComp = eachComp.PutSize()
#    p=mp.MPlot()
#    p.SetMainTitle("Fitting result")
#    p.SetYLabel("S(Q,E)")
#    p.SetXLabel("Hw")
#    p.AddData(result.Put(1))
#    p.AddData(result.Put(0))
#    for j in range(numComp):
#        p.AddData(eachComp.Put(j))
#    p.SetYScale()
#    p.SetPlotParam(1,mc="b",lc="b")
#    p.SetPlotParam(2,ls="-",lw=8,mk="",eb=False,lc="r") 
#    for j in range(numComp):
#        p.SetPlotParam(3+j,ls="--",lw=8,mk="",eb=False,lc="g")
#
#    p2=mp.MPlot()
#    p2.SetTraceNum(1)
#    p2.SetMainTitle("Residual Distribution")
#    p2.SetYLabel("(S(Q,E)$_{Obs}$ - S(Q,E)$_{Cal}$) / $\sigma$")
#    p2.SetXLabel("Hw")
#    p2.AddData(result.Put(3))
#    p2.SetPlotParam(1,mc="b",lc="b")
#
##########################################
#
#def showFitResult(result):
#    """
#    @param result          (ElementContainerArray/Matrix)
#    """
#    if type(result) == mm.ElementContainerArray:
#        return ShowFitResultEC(result)
#    if type(result) == mm.ElementContainerMatrix:
#        return ShowFitResultECA(result)
#
##########################################
#
#def showFitGuessECA(result):
#    """
#    @param result          (ElementContainerMatrix)
#    """
#    p=mp.MPlot()
#    p.SetMainTitle("initial Function")
#    p.SetYLabel("S(Q,E)")
#    p.SetXLabel("Hw")
#
#    p.SetTraceNum(2)
#    for i in range(result(0).PutSize()):
#        p.AddData(result.Put(1).Put(i))
#        p.AddData(result.Put(2).Put(i))
#        p.SetYScale()
#        p.SetPlotParam(i*2+1,mc="b",lc="b")
#        p.SetPlotParam(i*2+2,ls="-",lw=8,mk="",eb=False,lc="r") 
#        p.ChangePage(i+2)
#
#    p2=mp.MPlot()
#    p2.SetTraceNum(1)
#    p2.SetMainTitle("Residual Distribution(innitial)")
#    p2.SetYLabel("(S(Q,E)$_{Obs}$ - S(Q,E)$_{init}$) / $\sigma$")
#    p2.SetXLabel("Hw")
#    print (result.Put(3)).PutSize()
#    for i in range(result(0).PutSize()):
#        p2.AddData(result.Put(3).Put(i))
#        p2.SetPlotParam(i+1,mc="b",lc="b")
#        p2.ChangePage(i+1)
#
##########################################
#
#def showFitGuessEC(result):
#    """
#    @param result          (ElementContainerArray)
#    """
#    p=mp.MPlot()
#    p.SetMainTitle("initial function")
#    p.SetYLabel("S(Q,E)")
#    p.SetXLabel("Hw")
#    p.AddData(result.Put(1))
#    p.AddData(result.Put(2))
#    p.SetYScale()
#    p.SetPlotParam(1,mc="b",lc="b")
#    p.SetPlotParam(2,ls="-",lw=8,mk="",eb=False,lc="r") 
#
#    p2=mp.MPlot()
#    p2.SetTraceNum(1)
#    p2.SetMainTitle("Residual Distribution(innitial)")
#    p2.SetYLabel("(S(Q,E)$_{Obs}$ - S(Q,E)$_{init}$) / $\sigma$")
#    p2.SetXLabel("Hw")
#    p2.AddData(result.Put(3))
#    p2.SetPlotParam(1,mc="b",lc="b")
#
##########################################
#
#def showFitGuess(result):
#    """
#    @param result          (ElementContainerArray/Matrix)
#    """
#    if type(result) == mm.ElementContainerArray:
#        return ShowFitGuessEC(result)
#    if type(result) == mm.ElementContainerMatrix:
#        return ShowFitGuessECA(result)
#
##########################################
#def showFittingParameterPlot(result):
#    """
#    @param result          (ElementContainer)
#    """
#    hh=result.PutHeader()
#    if (hh.CheckKey("Axis")==0):
#        print "x-Axis label is not define" 
#    else:
#        Axis=result.PutHeader().PutString("Axis")
#    if (hh.CheckKey("Param")==0):
#        print "Parameter label is not define"
#        Param="" 
#    else:
#        Param=result.PutHeader().PutString("Param")
#    p=mp.MPlot()
#    p.SetMainTitle("Fitting Parameter result")
#    p.SetYLabel("Parameter "+Param)
#
#    if(Axis=="Q"):
#        p.SetXLabel("Q")
#    if(Axis=="Q2"):
#        p.SetXLabel("Q$^{2}$")
#
#    p.AddData(result)
#
##########################################

def LoadFittingParam(filename):
    """
    Load Initial Fitting Parameter
    @param filename  (str)
    """
    param_buf = []
    param     = []
    param_lb  = []
    param_ub  = []
    link      = []
    k=0
    function = None

    #flie read
    fi = open(filename,'r')
    for line in fi:
        if line[0]=="#":
            print("comment >> line[",k,"]",line)
        elif line[0:4]=="func":
            print("")
            print("function is set ",line.split(":")[1].strip("\n").strip("\r"))
            function = line.split(":")[1].strip("\n").strip("\r")
        else:
            param_buf.append(line.split(","))
        k += 1
    #print "number_of_list     \t=",len(param_buf)
    if len(param_buf[0])%4 ==0:
        NumParam = len(param_buf[0])/4
        #print "number_of_parameter \t=",NumParam
    else:
        print("file format is incorrect")
        print("file format is param1,param2,param3,....param1ub,param2ub,param3ub,,....param1lb,param2lb,param3lb")
        print("lb is lower boundary, ub is upper boundary")
    # create parameter strings-list
    for p in param_buf:
        p0=""
        for i in range(0,NumParam*4):#20130305 start
            if p[i] =="": 
                p[i]=" "             #20130305   end

        for i in range(0,NumParam):
            if p0=="":p0 = p[i].strip("\n").strip("\r")
            else:p0 = p0 + ","+ p[i].strip("\n").strip("\r")
        param.append(p0)
        p0=""
        for i in range(NumParam,NumParam*2):
            if p0=="":p0 = p[i].strip("\n").strip("\r")
            else:p0 = p0 + ","+ p[i].strip("\n").strip("\r")
        param_lb.append(p0)
        p0=""
        for i in range(NumParam*2,NumParam*3):
            if p0=="":p0 = p[i].strip("\n").strip("\r")
            else:p0 = p0 + ","+ p[i].strip("\n").strip("\r")
        param_ub.append(p0)
        p0=""
        for i in range(NumParam*3,NumParam*4):
            if p0=="":p0 = p[i].strip("\n").strip("\r")
            else:p0 = p0 + ","+ p[i].strip("\n").strip("\r")
        link.append(p0)

    print("")
    print("initial parameter")
    for i in range(len(param)):print("param",i," >> ",param[i])

    print("")
    print("upper boundary parameter")
    for i in range(len(param_ub)):print("param",i," >> ",param_ub[i])

    print("")
    print("lower boundary parameter")
    for i in range(len(param_lb)):print("param",i," >> ",param_lb[i])

    print("")
    print("link")
    for i in range(len(link)):print("param",i," >> ",link[i])

    return param,param_lb,param_ub,link,function

##########################################

def saveFittingParam(function,param,param_ub,param_lb,link,filename):
    """
    Save Initial Fitting Parameter
    @param function (str)
    @param param (str/list)
    @param param_ub (str/list)
    @param param_lb (str/list)
    @param link (str/list)
    @param filename  (str)
    """
    fw = open(filename,'w')
    fw.write("func:"+function+"\n")
    if type(param)==str and type(param_lb)==str and type(param_ub)==str and type(link)==str:
        fw.write(param+","+param_lb+","+param_ub+","+link)
        fw.close()
    if type(param)==list and type(param_lb)==list and type(param_ub)==list and type(link)==list:
        for i in range(len(param)):
            fw.write(param[i]+","+param_lb[i]+","+param_ub[i]+","+link[i]+"\n")
        fw.close()

##########################################

def exportCsvFitResult(result,filename):
    """
    export Fitting Result, function plotting data 
    @param result          (ElementContainerArray)
    @param filename        (str)
    """
    fw = open(filename,'w')

    if type(result) == mm.ElementContainerMatrix:

        inputECM=mm.ElementContainerMatrix()
        ecaDet=result.Put(1)
        inputEcaDet=mm.ElementContainerArray()
        for i in range(ecaDet.PutTableSize()):
            ecDet=ecaDet.Put(i)
            ecDet.SetKeys("xOrg","yOrg","eOrg")
            inputEcaDet.Add(ecDet)

        inputECM.Add(result.Put(0))
        inputECM.Add(inputEcaDet)
        inputECM.Add(result.Put(2))
        inputECM.Add(result.Put(3))
        inputECM.Add(result.Put(4))

        #eachComp = getFitEachResult(result)
        eachComp = getFitEachResult(inputECM)

        numComp = eachComp(0).PutSize()

        fit_result_eca = result.Put(0)
        num_of_Q = fit_result_eca.PutSize()
        observed_eca = result.Put(1)

        hh=fit_result_eca(0).PutHeaderPointer()
        func = ""  
        if (hh.CheckKey("FittingFunc")==1):
            func=hh.PutString("FittingFunc")
        funcName = getFunctionName(func)
        resultList = []
        headerText = ""

        for Q in range(num_of_Q):
            hh2=observed_eca(Q).PutHeaderPointer()
            xrange =[-100.0,100]
            if (hh.CheckKey("XRANGE")==1):
                xrange=hh2.PutDoubleVector("XRANGE")

            fit_result_x = []
            Q_min =[]
            Q_max =[]
            if "xOrg" in observed_eca.Put(Q).PutKeysList():
                observed_x = observed_eca.Put(Q).Put("xOrg")
                observed_y = observed_eca.Put(Q).Put("yOrg")
                observed_e = observed_eca.Put(Q).Put("eOrg")
                fitted_x = observed_eca.Put(Q).PutX()
                fitted_y = observed_eca.Put(Q).PutY()
                fitted_e = observed_eca.Put(Q).PutE()
            else:
                observed_x = observed_eca.Put(Q).PutX()
                observed_y = observed_eca.Put(Q).PutY()
                observed_e = observed_eca.Put(Q).PutE()

            #fit_result_y = fit_result_eca.Put(Q).PutY()
            for i in range(observed_y.size()):
                fit_result_x.append((observed_x[i]+observed_x[i+1])*0.5)
                Q_min.append(xrange[0])
                Q_max.append(xrange[1])

            resultList.append(Q_min)
            resultList.append(Q_max)
            resultList.append(fit_result_x)
            resultList.append(observed_y)
            resultList.append(observed_e)

            eachComp_eca = eachComp.Put(Q)
            totalComp_ec = eachComp_eca.Put(0)
            for i in range(numComp-1):
                totalComp_ec = totalComp_ec + eachComp_eca.Put(i+1)
            fit_result_y = totalComp_ec.PutY()
            #fit_result_y=[]
            #j=0
            #for i in range(observed_y.size()):
            #    if observed_y[i]==fitted_y[j]:
            #        fit_result_y.append(totalComp_ec.PutY()[j])
            #        if j<fitted_y.size()-1:
            #            j=j+1
            #    else:
            #        fit_result_y.append(0)
            resultList.append(fit_result_y)

            for i in range(numComp):
                eachComp_ec_y = eachComp_eca.Put(i).PutY()
                #eachComp_ec_y=[]
                #j=0
                #for k in range(observed_y.size()):
                #    if observed_y[k]==fitted_y[j]:
                #        eachComp_ec_y.append(eachComp_eca.Put(i).PutY()[j])
                #        if j<fitted_y.size()-1:
                #            j=j+1
                #    else:
                #        eachComp_ec_y.append(0)
                resultList.append(eachComp_ec_y)

            headerText += ("Qmin"+str(Q+1)+",")
            headerText += ("Qmax"+str(Q+1)+",")
            headerText += ("E(Q"+str(Q+1)+"),")
            headerText += ("I(Q"+str(Q+1)+"),")
            headerText += ("Isd(Q"+str(Q+1)+"),")
            headerText += ("Fitted(Q"+str(Q+1)+"),")
            for k in range(len(funcName)):
                headerText += (funcName[k]+"(Q"+str(Q+1)+"),")
        headerText = headerText.rstrip(',')

        num_text = len(resultList)
        fw.write(headerText+"\n")
        for j in range(observed_y.size()):
        #for j in range(len(resultList)):
            text = ""
            for i in range(num_text):
                #print "###",len(resultList[i])
                val=resultList[i][j]
                text += str(val)
                if i != (num_text - 1):
                    text += ","
            fw.write(text+"\n")

    if type(result) == mm.ElementContainerArray:
        eachComp = getFitEachResult(result)
        numComp = eachComp.PutSize()

        observed_ec = result.Put(1)
        hh=fit_result_ec.PutHeaderPointer()
        hh2=observed_ec.PutHeaderPointer()
        func = ""
        xrange =[-100.0,100]
        if (hh.CheckKey("FittingFunc")==1):
            func=hh.PutString("FittingFunc")
        if (hh.CheckKey("XRANGE")==1):
            xrange=hh2.PutDoubleVector("XRANGE")

        funcName = getFunctionName(func)
        headerText = ""
        resultList = []
        fit_result_x_buf = fit_result_ec.PutX()
        fit_result_x = []
        Q_min =[]
        Q_max =[]
        if "xOrg" in observed_eca.Put(Q).PutKeysList():
            observed_x = observed_ec.Put("xOrg")
            observed_y = observed_ec.Put("yOrg")
            observed_e = observed_ec.Put("eOrg")
        else:
            observed_x = observed_ec.PutX()
            observed_y = observed_ec.PutY()
            observed_e = observed_ec.PutE()

        fit_result_y = fit_result_ec.PutY()
        for i in range(observed_y.size()):
            fit_result_x.append((observed_x[i]+observed_x[i+1])*0.5)
            Q_min.append(xrange[0])
            Q_max.append(xrange[1])

        resultList.append(Q_min)
        resultList.append(Q_max)
        resultList.append(fit_result_x)
        resultList.append(observed_y)
        resultList.append(observed_e)

        eachComp_eca = eachComp
        totalComp_ec = eachComp_eca.Put(0)
        for i in range(numComp-1):
            totalComp_ec = eachComp_eca.Put(i+1)
        fit_result_y = totalComp_ec.PutY()
        resultList.append(fit_result_y)

        for i in range(numComp):
            eachComp_ec_y = eachComp_eca.Put(i).PutY()
            resultList.append(eachComp_ec_y)

        headerText += ("Qmin,")
        headerText += ("Qmax,")
        headerText += ("E,")
        headerText += ("I,")
        headerText += ("Isd,")
        headerText += ("Fitted,")
        for k in range(len(funcName)):
            headerText += (funcName[k]+",")
        headerText = headerText.rstrip(',')
        fw.write(headerText+"\n")
        num_text = len(resultList)
        for j in range(fit_result_y.size()):
            text = ""
            for i in range(num_text):
                val=resultList[i][j]
                text += str(val)
                if i != (num_text - 1):
                    text += ","
            fw.write(text+"\n")

    fw.close()

##########################################

def exportCsvFitParamResult(result,filename):
    """
    export Fitting Result, fitting parameter data 
    @param result          (ElementContainerArray)
    @param filename        (str)
    """
    fw = open(filename,'w')

    if type(result) == mm.ElementContainerMatrix:
        fit_result_eca = result.Put(0)
        num_of_Q = fit_result_eca.PutSize()
        hh=fit_result_eca(0).PutHeaderPointer()
        if (hh.CheckKey("FittingFunc")==1):
            func = hh.PutString("FittingFunc")
            paramList = getParamName(func)
            Text = "Q,"
            for textList in paramList:
                Text += textList
                Text += ",,"
            Text = Text.rstrip(",")
            fw.write(Text+"\n")
        for i in range(num_of_Q):
            hh=fit_result_eca(i).PutHeaderPointer()
            Text = ""

            if (hh.CheckKey("XRANGE")==1):
                XRange=hh.PutDoubleVector("XRANGE")
                Q = (XRange[0]+XRange[1])*0.5
            else:
                Q = 0
            Text += str(Q)+","
            flug = True
            if (hh.CheckKey("ChiSq")==1):
                chisq = hh.PutDouble("ChiSq")
                print(chisq,chisq!=chisq)
                if chisq==chisq:
                    pnum = 0
                    while flug:
                        if (hh.CheckKey("FittingParam"+str(pnum))==1):
                            Text += str(hh.PutDouble("FittingParam"+str(pnum)))+","
                            Text += str(hh.PutDouble("FittingParamErr"+str(pnum)))+","
                            pnum += 1
                        if (hh.CheckKey("FittingParam"+str(pnum))==0):
                            Text = Text.rstrip(",")
                            flug = False
                    if hh.PutDouble("FittingParamErr0")!=0.0:
                        fw.write(Text+"\n")
                elif chisq!=chisq:
                    pnum = 0
                    while flug:
                        if (hh.CheckKey("FittingParam"+str(pnum))==1):
                            Text += "0,0,"
                            pnum += 1
                        if (hh.CheckKey("FittingParam"+str(pnum))==0):
                            Text = Text.rstrip(",")
                            flug = False
            #fw.write(Text+"\n")

    if type(result) == mm.ElementContainerArray:
        fit_result_ec = result.Put(0)
        hh=fit_result_ec.PutHeaderPointer()
        if (hh.CheckKey("FittingFunc")==1):
            func = hh.PutStrinng("FittingFunc")
            paramList = getParamName(func)
            Text = "Q,"
            for textList in paramlist:
                Text += textList
                Text += ","
            Text = Text.rstrip(",")
            fw.write(Text+"\n")
        Text = ""
        if (hh.CheckKey("XRANGE")==1):
            XRange=hh.PutDoubleVector("XRANGE")
            Q = (XRange[0]+XRange[1])*0.5
        else:
            Q = 0
        flug = True
        Text += str(Q)
        if (hh.CheckKey("ChiSq")==1):
            chisq = hh.PutDouble("ChiSq")
            if chisq==chisq:
                pnum = 0
                while flug:
                    if (hh.CheckKey("FittingParam"+str(pnum))==1):
                        Text += str(hh.PutDouble("FittingParam"+str(pnum)))
                        Text += str(hh.PutDouble("FittingParamErr"+str(pnum)))
                        pnum += 1
                    if (hh.CheckKey("FittingParam"+str(pnum))==0):
                        Text = Text.rstrip(",")
                        flug = False
                if hh.PutDouble("FittingParamErr0")!=0.0:
                    fw.write(Text+"\n")
            elif chisq!=chisq:
                pnum = 0
                while flug:
                    if (hh.CheckKey("FittingParam"+str(pnum))==1):
                        Text += "0,0,"
                        pnum += 1
                    if (hh.CheckKey("FittingParam"+str(pnum))==0):
                        Text = Text.rstrip(",")
                        flug = False
        #fw.write(Text+"\n")
    fw.close()

##########################################
def getFunctionName(function):
    """
    Fitting関数の省略形表記の結合の入力文字列から､複数回関数が選択されている場合関数名にその数字を付けたListとして返す。
    また、一回のみの選択でも1を付けて入力文字列から関数名＋数字のListを返す
    @param function          (string)
    """
    function=replacePlusStr(function)
    h=function.split()
    funclist =[]
    funclist.append(["lc","lorentzConv"])
    funclist.append(["gc","gaussConv"])
    funclist.append(["dc","deltaConv"])
    funclist.append(["pv1c","pseudoVoigt1Conv"])
    funclist.append(["pv2c","pseudoVoigt2Conv"])
    funclist.append(["dhoc","dampedHarmonicOscillatorConv"])
    funclist.append(["alc","augmentedLorentzianConv"])
    funclist.append(["cc","constantConv"])
    funclist.append(["p1c","polynomial1Conv"])
    funclist.append(["p2c","polynomial2Conv"])
    funclist.append(["p3c","polynomial3Conv"])
    funclist.append(["l","lorentz"])
    funclist.append(["g","gauss"])
    funclist.append(["pv1","pseudoVoigt1"])
    funclist.append(["pv2","pseudoVoigt2"])
    funclist.append(["dho","dampedHarmonicOscillator"])
    funclist.append(["al","augmentedLorentzian"])
    funclist.append(["c","constant"])
    funclist.append(["p1","polynomial1"])
    funclist.append(["p2","polynomial2"])
    funclist.append(["p3","polynomial3"])

    h_list = []
    outputText = ""
    output = []
    for i in range(len(h)):
        h_list.append(h[i])
        numTag = h_list.count(h[i])
        funcName = h[i]
        for j in range(len(funclist)):
            if h[i]==funclist[j][0]:
                funcName = funclist[j][1]
        output.append(funcName+str(numTag))

    return output

##########################################

def getParamName(function):
    """
    Fitting関数の省略形表記の結合の入力文字列から､その関数に対応するパラメータ名を結合した関数名のListを返す。関数名_パラメータ名
    @param function          (string)
    """
    function=replacePlusStr(function)
    h=function.split()
    funclist =[]
    funclist.append(["lc","lorentzConv",["Area","Position","HWHM"]])
    funclist.append(["gc","gaussConv",["Area","Position","HWHM"]])
    funclist.append(["dc","deltaConv",["Area","Position"]])
    funclist.append(["pv1c","pseudoVoigt1Conv",["Area","Position","HWHM","LinearCombinationCoefficient"]])
    funclist.append(["pv2c","pseudoVoigt2Conv",["Area","Position","HWHMofGauss","HWHMofLorentz","LinearCombinationCoefficient"]])
    funclist.append(["dhoc","dampedHarmonicOscillatorConv",["Area","Frequency","FrictionCoefficient"]])
    funclist.append(["alc","augmentedLorentzianConv",["Area","Position","Width","Power"]])
    funclist.append(["cc","constantConv",["Constant"]])
    funclist.append(["p1c","polynomial1Conv",["PolynomialCoefficient"]])
    funclist.append(["p2c","polynomial2Conv",["PolynomialCoefficient"]])
    funclist.append(["p3c","polynomial3Conv",["PolynomialCoefficient"]])
    funclist.append(["l","lorentz",["Area","Position","HWHM"]])
    funclist.append(["g","gauss",["Area","Position","HWHM"]])
    funclist.append(["pv1","pseudoVoigt1",["Area","Position","HWHM","LinearCombinationCoefficient"]])
    funclist.append(["pv2","pseudoVoigt2",["Area","Position","HWHM","LinearCombinationCoefficient"]])
    funclist.append(["dho","dampedHarmonicOscillator",["Area","Frequency","FrictionCoefficient"]])
    funclist.append(["al","augmentedLorentzian",["Area","Position","Width","Power"]])
    funclist.append(["c","constant",["Constant"]])
    funclist.append(["p1","polynomial1",["PolynomialCoefficient"]])
    funclist.append(["p2","polynomial2",["PolynomialCoefficient"]])
    funclist.append(["p3","polynomial3",["PolynomialCoefficient"]])

    h_list = []
    outputText = ""
    output = []
    for i in range(len(h)):
        h_list.append(h[i])
        numTag = h_list.count(h[i])
        funcName = h[i]
        for j in range(len(funclist)):
            if h[i]==funclist[j][0]:
                funcName = funclist[j][1]
                for ll in funclist[j][2]:
                    output.append(funcName+str(numTag)+"_"+ll)

    return output

##########################################

def selectQbin(eca,num):
    """
    入力したECAから指定したECを返す
    @param eca          (ElementContainerArray)
    @param num          (int)
    @retval ec          (ElementContainer)
    """
    if type(eca)==mm.ElementContainer:
        print("input data is ElementContainer")
        return eca
    if type(eca)==mm.ElementContainerArray:
        ecaNum = eca.PutSize()
        if num<0 or num > ecaNum:
            print("input num is out of range")
        else:
            return eca.Put(num)
    if type(eca)==mm.ElementContainerMatrix:
        print("input data is ElementContainerMatrix")
        ec = mm.ElementContainer()
        return ec

##########################################

def chceckZeroYValueForECA(eca):
    """
    ECA内のECの強度0のビンを探しそれを削除する
    @param  eca         (ElementContainerArray)
    @retval ecaNew      (ElementContainerArray)
    """
    ecaNew = mm.ElementContainerArray()
    ecaNew.InputHeader(eca.PutHeader())
    flag = False
    for i in range(eca.PutTableSize()):
        ec = eca.Put(i)
        ecX = ec.PutX()
        ecY = ec.PutY()
        ecE = ec.PutE()

        ecXnewCenter=[]
        ecXnew=[ecX[0]]
        ecYnew=[]
        ecEnew=[]
        for j in range(ecE.size()):
            if ecE[j]!=0.0:
                ecXnewCenter.append((ecX[j]+ecX[j+1])*0.5)
                ecYnew.append(ecY[j])
                ecEnew.append(ecE[j])
        for k in range(len(ecXnewCenter)):
            ecXnewElement= - ecXnew[k] + ecXnewCenter[k]*2
            ecXnew.append(ecXnewElement)

        if len(ecXnew)!=ecX.size():
            #print "eca("+str(i)+") has-zero-event-bin-data"
            #print "eca("+str(i)+"), zero-event-bins were deleted"
            flag=True
        if len(ecYnew)==0:
            ecYnew=[0]*ecY.size()
            ecEnew=[1]*ecE.size()
            ecXnew=ecX
            flag=True
            #print "eca("+str(i)+") has zero-event-bin-data"

        ecNew = mm.ElementContainer()
        #print ecXnew
        #print min(ecXnew),max(ecXnew)
        ecNew.Add("x",ecXnew)
        ecNew.Add("y",ecYnew)
        ecNew.Add("e",ecEnew)
        ecNew.Add("xOrg",ecX)
        ecNew.Add("yOrg",ecY)
        ecNew.Add("eOrg",ecE)
        ecNew.SetKeys("x","y","e")
        ecNew.InputHeader(ec.PutHeader())
        ecaNew.Add(ecNew)

    if flag:
        print("eca has zero-event-data")

    return ecaNew

##########################################

def replacePlusStr(text):
    """
    +文字列を空白文字列にし、連続した空白文字列をひとつの空白文字列にする。
    @param text        (string)
    @retval textOut    (string)
    """
    while 1:
        text = text.replace("+"," ")
        text = text.replace("  "," ")
        if text.find("  ") == -1:
            break
    return text

##########################################

def ElementContainerToXYList(ec):
    """
    入力したElementContainerからx,yの数値データのListを返す
    @param ec        (ElementContainer)
    @retval xylist   (list)
    """
    xbin = ec.PutX()
    ybin = ec.PutY()
    xbinList = []
    ybinList = []
    numYbin = ybin.size()
    for i in range(numYbin):
        xbinList.append((xbin[i]+xbin[i+1])*0.5)
        ybinList.append(ybin[i])
    xyList = [xbinList,ybinList]
    return xyList

##########################################

def ElementContainerToXYEList(ec):
    """
    @param ec        (ElementContainer)
    入力したElementContainerからx,y,eの数値データのListを返す
    @retval xylist   (list)
    """
    xbin = ec.PutX()
    ybin = ec.PutY()
    ebin = ec.PutE()
    xbinList = []
    ybinList = []
    ebinList = []
    numYbin = ybin.size()
    for i in range(numYbin):
        xbinList.append((xbin[i]+xbin[i+1])*0.5)
        ybinList.append(ybin[i])
        ebinList.append(ebin[i])
    xyList = [xbinList,ybinList,ebinList]
    return xyList

##########################################

def ElementContainerArrayToXYList2(eca):
    """
    入力したElementContainerArrayからx,yの数値データのListをそれぞれのECで計算し、それらを満たしたListを返す
    @param eca        (ElementContainerArray)
    @retval xylist   (list)
    """
    numOfEc = eca.PutTableSize()
    xyList2 = []

    for i in range(numOfEc):
        ec = eca.Put(i)
        xyList = ElementContainerToXYList(ec)
        xyList2.append(xyList)

    return xyList2

##########################################

def ElementContainerArrayToXYEList2(eca):
    """
    入力したElementContainerArrayからx,y,eの数値データのListをそれぞれのECで計算し、それらを満たしたListを返す
    @param eca        (ElementContainerArray)
    @retval xylist   (list)
    """
    numOfEc = eca.PutTableSize()
    xyList2 = []

    for i in range(numOfEc):
        ec = eca.Put(i)
        xyList = ElementContainerToXYEList(ec)
        xyList2.append(xyList)

    return xyList2

##########################################

def getNumOfEC(eca):
    """
    ECAに含まれるECの数を返す
    @param eca        (ElementContainerArray)
    @retval val       (int)
    """
    numOfEc = eca.PutTableSize()
    return numOfEc

##########################################

def getUnit(eca):
    """
    ECAに含まれるECのそれぞれの単位をListで返す
    @param eca        (ElementContainerArray)
    @retval val       (list)
    """
    numOfEc = eca.PutTableSize()
    val=[]
    for i in range(numOfEc):
        ec=eca.Put(i)
        val.append([ec.PutUnit(ec.PutXKey()),ec.PutUnit(ec.PutYKey()),ec.PutUnit(ec.PutEKey())])

    return val

##########################################

def getKeyName(eca):
    """
    ECAに含まれるECのそれぞれのKey名をListで返す
    @param eca        (ElementContainerArray)
    @retval val       (list)
    """
    numOfEc = eca.PutTableSize()
    val=[]
    for i in range(numOfEc):
        ec=eca.Put(i)
        val.append([ec.PutXKey(),ec.PutYKey(),ec.PutEKey()])

    return val

##########################################

def getQBoundary(eca):
    """
    ECAに含まれるECのそれぞれのQの境界をListで返す
    @param eca        (ElementContainerArray)
    @retval qlist     (list)
    """
    qlist=[]
    if eca != None:
        numOfEc = eca.PutTableSize()
        for i in range(numOfEc):
            ec=eca.Put(i)
            hh=ec.PutHeader()
            if hh.CheckKey("XRANGE")==0:
                qlist.append(i)
            else:
                xr=hh.PutDoubleVector("XRANGE")
                if i==0:
                    qlist.append(xr[0])
                qlist.append(xr[1])
    return qlist

##########################################

def getQValue(eca):
    """
    入力したECAのECのヘッダーXRANGEから中心値をQとして計算しそのListを返す
    @param result          (ElementContainerMatrix)
    """
    val=[]
    num=eca.PutTableSize()
    for i in range(num):
        hh=eca.Put(i).PutHeader()
        XRange=hh.PutDoubleVector("XRANGE")   #20130124 add-line, start
        CenterVal = (XRange[0]+XRange[1])*0.5
        val.append(CenterVal)
    return val

##########################################

def setConvolutionRange(eca,rangelist):
    """
    入力したECAに対応した畳み込積分範囲のListからECAないのECのヘッダーに情報を書き込む
    @param eca        (ElementContainerArray)
    @param rangelist  (list)
    @retval ecaOut       (ElementContainerArray)
    """
    ecaOut=mm.ElementContainerArray()
    numOfEc = eca.PutTableSize()
    for i in range(numOfEc):
        ec=eca.Put(i)
        hh=ec.PutHeader()
        if (hh.CheckKey("CONVOLRANGE")==0):
            hh.Add("CONVOLRANGE",mm.ListToDoubleVector(rangelist[i]))
        if (hh.CheckKey("CONVOLRANGE")==1):
            hh.OverWrite("CONVOLRANGE",mm.ListToDoubleVector(rangelist[i]))
        ec.InputHeader(hh)
        ecaOut.Add(ec)
    return ecaOut

##########################################

def getBinInformation(eca):
    """
    入力したECAの先頭のECのX軸初期値、最終値、刻み幅のタプルを返す
    @param eca        (ElementContainerArray)
    @retval val       (list)
    """
    if type(eca)==mm.ElementContainerArray:
        ecX = eca.Put(0).PutX()
        return ecX[0],ecX[-1],ecX[1]-ecX[0]
    else:
        return None,None,None

##########################################

def getFittingRawData(eca):
    """
    ECAの数値データを変えす。x,y
    @param eca        (ElementContainerArray)
    @retval xylist   (list)
    """
    if type(eca)==mm.ElementContainerArray:
        xylist = ElementContainerArrayToXYList2(eca)
    else:
        print("filetype is incorrect.")
        print("format is ElementContainerArray")
        xylist=[]
    return xylist

##########################################

def getFittingRawDataWithError(eca):
    """
    ECAの数値データを変えす。x,y,e
    @param eca        (ElementContainerArray)
    @retval xylist   (list)
    """
    if type(eca)==mm.ElementContainerArray:
        xylist = ElementContainerArrayToXYEList2(eca)
    else:
        print("filetype is incorrect.")
        print("format is ElementContainerArray")
        xylist=[]
    return xylist

##########################################

def getFittingResultData(ecm):
    """
    Fitting結果のECMよりFitting結果の関数の数値データを返すx,y
    @param ecm        (ElementContainerMatrix)
    @retval paraxylist   (list)
    """
    paraxylist = []
    if type(ecm)==mm.ElementContainerMatrix:
        eachECM = getFitEachResult(ecm)
        numOfEc = eachECM.PutTableSize()
        for i in range(numOfEc):
            #ecm.Put(1).Put(i).Dump()
            eca = eachECM.Put(i)
            #ecTot = (ecm.Put(0)).Put(i) # from levmar loutine result

            numFunc = eca.PutTableSize()
            ecTot=eca.Put(0)
            if numFunc > 1:
                for i in range(numFunc-1):
                    ecTot = ecTot + eca.Put(i+1) # sum of function from fitting parameter result

            xylist_org = ElementContainerToXYList(ecTot)
            xylist2 = ElementContainerArrayToXYList2(eca)
            xylist = [xylist_org]
            xylist.extend(xylist2)
            paraxylist.append(xylist)
    else:
        print("filetype is incorrect.")
        print("format is ElementContainerMatrix")

    return paraxylist

##########################################

def getResidualDistributionData(ecm):
    """
    Fitting結果のECMより残渣の数値データを返すx,y
    @param ecm        (ElementContainerMatrix)
    @retval xylist   (list)
    """
    if type(ecm)==mm.ElementContainerMatrix and ecm(0).PutTableSize>=3:
        eca = ecm.Put(3)
        xylist = ElementContainerArrayToXYList2(eca)
    else:
        print("filetype is incorrect.")
        print("format is ElementContainerMatrix including with fitting-results")
        xylist=[]
    return xylist

##########################################

def getDeltaFuncResolECA(ecaIn):
    """
    入力したECAのヒストグラムと同じX軸を持ったデルタ関数のECAを返す
    @param ecaIn        (ElementContainerArray)
    @retval ecaOut      (ElementContainerArray)
    """
    ecaOut=mm.ElementContainerArray()
    num_of_eca=ecaIn.PutTableSize()

    for i in range(num_of_eca):
        ecX=ecaIn(i).PutX()
        abs_val=abs(ecX[0])
        zero_bin=0

        for i in range(ecX.size()):
            if abs(ecX[i])<abs_val:
                abs_val=abs(ecX[i])
                zero_bin=i

        ecY=[0]*(ecX.size()-2)
        ecY.insert(zero_bin,1)

        ec=mm.ElementContainer()
        ec.Add("x",ecX)
        ec.Add("y",ecY)
        ec.SetKeys("x","y","y")

        ecaOut.Add(ec)

    return ecaOut

##########################################

def usePastExecuteResult(ecmIn,num,resultlist):
    """
    入力したFitting結果を、指定したECのヘッダーに書き込む
    @param ecmIn        (ElementContainerArray)
    @param num          (int)
    @param resultlist   (list)
    @retval None
    """
    ecmOut=mm.ElementContainerMatrix()
    result_eca=mm.ElementContainerArray()
    residual_eca=mm.ElementContainerArray()
    ecaDet=mm.ElementContainerArray()

    ecaDet_org=ecmIn.Put(1)
    result_eca_org=ecmIn.Put(0)
    residual_eca_org=ecmIn.Put(3)

    param=""
    for p in resultlist[0]:
        if param=="":
            param = str(p)
        else:
            param = param+","+str(p)

    function= ecaDet_org(num).PutHeader().PutString("FittingFunc")

    for i in range(ecmIn.Put(1).PutTableSize()):

        if i!=num:
            result_eca.Add(result_eca_org(i))
            residual_eca.Add(residual_eca_org(i))
            ecaDet.Add(ecaDet_org(i))
        else:
            result_ec = PF.getFittingFunction(function,param,ecaDet_org(num))
            result_ec.SetUnit(result_ec.PutXKey(),'meV')
            result_ec.SetUnit(result_ec.PutYKey(),'')

            hh = ecaDet_org(num).PutHeader()

            for i in range(len(resultlist[0])):
                if (hh.CheckKey("FittingParam"+str(i))==0):
                    hh.Add("FittingParam"+str(i),resultlist[0][i])
                if (hh.CheckKey("FittingParam"+str(i))==1):
                    hh.OverWrite("FittingParam"+str(i),resultlist[0][i])

                if (hh.CheckKey("FittingParamErr"+str(i))==0):
                    hh.Add("FittingParamErr"+str(i),resultlist[1][i])
                if (hh.CheckKey("FittingParamErr"+str(i))==1):
                    hh.OverWrite("FittingParamErr"+str(i),resultlist[1][i])

            result_ec.InputHeader(hh)
            ecDet=ecaDet_org(num)
            ecDet.InputHeader(hh)

            residual_ec = PF.getResidualDistribution(ecaDet_org(num),result_ec)
            residual_ec.SetUnit(residual_ec.PutXKey(),'meV')
            residual_ec.SetUnit(residual_ec.PutYKey(),'')
            result_eca.Add(result_ec)
            residual_eca.Add(residual_ec)
            ecaDet.Add(ecDet)

    ecmOut.Add(result_eca)
    ecmOut.Add(ecaDet)
    ecmOut.Add(ecmIn.Put(2))
    ecmOut.Add(residual_eca)
    ecmOut.Add(ecmIn.Put(4))

    residual_eca = getResidualDistribution(ecmOut)
    ecmOut.EraseElement(4)
    ecmOut.EraseElement(3)
    ecmOut.Add(residual_eca)
    ecmOut.Add(ecmIn.Put(4))

    return ecmOut

##########################################

def importDaveFormat(filepath):
    """
    Dave形式のヒストグラムをElementContainerに変換
    @param filepath          (str)
    @retval eca      (ElementContainerArray)
    """
    return DFB.importDaveFormat(filepath)

##########################################

def exportDaveFormat(eca,filepath):
    """
    ElementContainerをDave形式のヒストグラムに変換
    @param eca           (ElementContainerArray)
    @param filepath          (str)
    @retval None
    """
    return DFB.exportDaveFormat(eca,filepath)

##########################################

def OutputECText(ec,filename):
    """
    ElementContainerをText形式のデータに変換
    @param ec      (ElementContainer)
    @param filename          (string)
    @retval None
    """
    return DFB.OutputECText(ec,filename)

##########################################

def ReadECA(filename):
    """
    c++-boostでシリアライズされたECAを読み込む
    @param filename  (string)
    @retval eca      (ElementContainerArray)
    """
    return DFB.ReadECA(filename)

##########################################

def OutputECA(eca,filepath):
    """
    ECAをc++-boostでシリアライズする
    @param eca           (ElementContainerArray)
    @param filepath      (str)
    @retval None
    """
    return DFB.OutputECA(eca,filepath)

##########################################

def importTextFormat(filepath):
    """
    Text形式のElementContainerArrayデータに変換
    @param filepath      (str)
    @retval eca          (ElementContainerArray)
    """
    return DFB.importTextFormat(filepath)

##########################################

def exportTextFormat(eca,filepath):
    """
    ElementContainerArrayをText形式のデータに変換
    @param eca           (ElementContainerArray)
    @param filepath      (str)
    @retval None
    """
    return DFB.exportTextFormat(eca,filepath)

##########################################

def getResidualDistribution(ecm):

    paraxylist = []
    if type(ecm)==mm.ElementContainerMatrix:
        eachECM = getFitEachResult(ecm)
        numOfEc = eachECM.PutTableSize()
        ecaOut= mm.ElementContainerArray()
        ecaDet=ecm.Put(1)
        for i in range(numOfEc):
            eca = eachECM.Put(i)
            ecDet=ecaDet.Put(i)
            numFunc = eca.PutTableSize()
            ecTot=eca.Put(0)
            if numFunc > 1:
                for i in range(numFunc-1):
                    ecTot = ecTot + eca.Put(i+1)
            #xvec=eraseLastVect(ecTot.PutX())
            #yvec=eraseLastVect(ecTot.PutY())
            #evec=eraseLastVect(ecTot.PutE())
            #ecTot.Add("xnew",xvec)
            #ecTot.Add("ynew",yvec)
            #ecTot.Add("enew",evec)
            #ecTot.SetKeys("xnew","ynew","enew")

            #if "xOrg" in ecDet.PutKeysList():
            #    ecDet.SetKeys("xOrg","yOrg","eOrg")
            #if "x" in ecDet.PutKeysList():
            #    ecDet.SetKeys("x","y","e")

            ecRes=PF.getResidualDistribution(ecDet,ecTot)

            #ecDet.Dump()
            #ecTot.Dump()
            #ecRes.Dump()


            ecaOut.Add(ecRes)
    else:
        print("filetype is incorrect.")
        print("format is ElementContainerMatrix")

    return ecaOut

def eraseLastVect(vec):
    ret=[vec[i] for i in range(vec.size()-1)]
    return ret

##########################################

def getTotalResultDistribution(ecm):

    paraxylist = []
    if type(ecm)==mm.ElementContainerMatrix:
        eachECM = getFitEachResult(ecm)
        numOfEc = eachECM.PutTableSize()
        ecaOut= mm.ElementContainerArray()
        ecaDet=ecm.Put(1)
        ecaResult=ecm.Put(0)
        for i in range(numOfEc):
            eca = eachECM.Put(i)
            ecDet=ecaDet.Put(i)
            ecResultHeader=ecaResult.Put(i).PutHeader()
            numFunc = eca.PutTableSize()
            ecTot=eca.Put(0)
            if numFunc > 1:
                for i in range(numFunc-1):
                    ecTot = ecTot + eca.Put(i+1)

            ecTot.SetUnit(ecTot.PutXKey(),'meV')
            ecTot.SetUnit(ecTot.PutYKey(),'')
            ecTot.InputHeader(ecResultHeader)
            ecaOut.Add(ecTot)
    else:
        print("filetype is incorrect.")
        print("format is ElementContainerMatrix")

    return ecaOut



import utsusemi.DNA.ana.Reduction.Histogram as HH
import Manyo.DNA as md
import Manyo as mm
import numpy
import scipy
import scipy.stats
import math
import os
import multiprocessing
import time
#-------------------------------------------------------------------------------------------------
def GetAnalysisParameter(laticeSpace=3.134600):
    # LaticeSpace dependence
    params=[0.0   ,    1.0,      0.1, 0.0, 0.0, 0.0, laticeSpace, 0.000000]

    return HH.GetAnalysisParameter(laticeSpace)
#-------------------------------------------------------------------------------------------------
def CheckFitFuncDegree(eiArray,degree):

    if len(eiArray)>degree:
        return True
    else:
        print "This function requires the following conditions."
        print " ( initial Energy Array Length > degree )"
        return False
#-------------------------------------------------------------------------------------------------
def CalcASS(eiArray, sampleFileName ,degree, output, laticeSpace=3.134600, alpha=1.0):

    if os.path.exists(output):
        os.remove(output)

    ef,aa,pa=GetAnalysisParameter(laticeSpace)

    #psdn   = 224
    psdn   = len(ef)
    #psdn   = 1

    pixeld = 60

    pixeln=len(ef[0])/pixeld
    #pixeln = 6
    #pixeln = 1

    if CheckFitFuncDegree(eiArray,degree):
        print "\n0/"+str(psdn*pixeln*pixeld)+"\n"
        ptime=time.time()
        for i in range(psdn):
            for j in range(pixeln):
                psd   = [i]
                pixel = range(j*pixeld,(j+1)*pixeld)
                #CalcACoeffArray(psd, pixel, eiArray, sampleFileName ,degree, output, "ASS", ef, pa, aa, alpha)
                CalcACoeffArrayMulth(psd, pixel, eiArray, sampleFileName ,degree, output, "ASS", ef, pa, aa, alpha)
                print "\n" + str((j+1) * pixeld + i * pixeln * pixeld)+"/" + str(psdn * pixeln * pixeld) + "\n"
                ntime=time.time()
                print ntime-ptime
                ptime=ntime
#-------------------------------------------------------------------------------------------------
def CalcACSC(eiArray, sampleFileName ,degree, output, laticeSpace=3.134600, alpha=1.0):
    if os.path.exists(output):
        os.remove(output)

    ef,aa,pa=GetAnalysisParameter(laticeSpace)

    #psdn   = 224
    psdn    = len(ef)
    #psdn   = 1
    pixeld  = 60
    pixeln  = len(ef[0])/pixeld
    #pixeln = 6
    #pixeln = 1

    if CheckFitFuncDegree(eiArray,degree):
        print "\n0/"+str(psdn*pixeln*pixeld)+"\n"
        ptime=time.time()
        for i in range(psdn):
            for j in range(pixeln):
                psd   = [i]
                pixel = range(j*pixeld,(j+1)*pixeld)
                #CalcACoeffArray(psd, pixel, eiArray, sampleFileName ,degree, output, "ACSC", ef, pa, aa, alpha)
                CalcACoeffArrayMulth(psd, pixel, eiArray, sampleFileName ,degree, output, "ACSC", ef, pa, aa, alpha)
                print "\n" + str((j+1) * pixeld + i * pixeln * pixeld)+"/" + str(psdn * pixeln * pixeld) + "\n"
                ntime=time.time()
                print ntime-ptime
                ptime=ntime
#-------------------------------------------------------------------------------------------------
def CalcACSS(eiArray, sampleFileName ,degree, output, laticeSpace=3.134600, alpha=1.0):
    if os.path.exists(output):
        os.remove(output)

    ef,aa,pa=GetAnalysisParameter(laticeSpace)

    #psdn   = 224
    psdn    = len(ef)
    #psdn   = 1
    pixeld  = 60
    pixeln  = len(ef[0])/pixeld
    #pixeln = 6
    #pixeln = 1

    if CheckFitFuncDegree(eiArray,degree):
        print "\n0/"+str(psdn*pixeln*pixeld)+"\n"
        ptime=time.time()
        for i in range(psdn):
            for j in range(pixeln):
                psd   = [i]
                pixel = range(j*pixeld,(j+1)*pixeld)
                #CalcACoeffArray(psd, pixel, eiArray, sampleFileName ,degree, output, "ACSS", ef, pa, aa, alpha)
                CalcACoeffArrayMulth(psd, pixel, eiArray, sampleFileName ,degree, output, "ACSS", ef, pa, aa, alpha)
                print "\n" + str((j+1) * pixeld + i * pixeln * pixeld)+"/" + str(psdn * pixeln * pixeld) + "\n"
                ntime=time.time()
                print ntime-ptime
                ptime=ntime
#-------------------------------------------------------------------------------------------------
def CalcACC(eiArray, sampleFileName ,degree, output, laticeSpace=3.134600, alpha=1.0):
    if os.path.exists(output):
        os.remove(output)

    ef,aa,pa=GetAnalysisParameter(laticeSpace)

    #psdn   = 224
    psdn    = len(ef)
    #psdn   = 1
    pixeld  = 60
    pixeln  = len(ef[0])/pixeld
    #pixeln = 6
    #pixeln = 1

    if CheckFitFuncDegree(eiArray,degree):
        print "\n0/"+str(psdn*pixeln*pixeld)+"\n"
        ptime=time.time()
        for i in range(psdn):
            for j in range(pixeln):
                psd   = [i]
                pixel = range(j*pixeld,(j+1)*pixeld)
                #CalcACoeffArray(psd, pixel, eiArray, sampleFileName ,degree, output, "ACC", ef, pa, aa, alpha)
                CalcACoeffArrayMulth(psd, pixel, eiArray, sampleFileName ,degree, output, "ACC", ef, pa, aa, alpha)
                print "\n" + str((j+1) * pixeld + i * pixeln * pixeld)+"/" + str(psdn * pixeln * pixeld) + "\n"
                ntime=time.time()
                print ntime-ptime
                ptime=ntime
#-------------------------------------------------------------------------------------------------
def CalcASSOfElastic(sampleFileName, output, laticeSpace=3.134600, alpha=1.0):
    if os.path.exists(output):
        os.remove(output)

    ef,aa,pa=GetAnalysisParameter(laticeSpace)
    psdn   = len(ef)
    #psdn    = 1
    pixeln = len(ef[0])

    for i in range(psdn):
    #for i in range(55,60):
        psd = [i]
        pixel = range(pixeln)
        CalcACoeffOfElasticArrayMulth(psd, pixel, sampleFileName, output, "ASS", ef, pa, aa)
        print "\n" + str( (i+1) * pixeln)+"/" + str(pixeln * psdn) + "\n"
#-------------------------------------------------------------------------------------------------
def CalcACoeffArray(psd, pixel, eiArray, sampleFileName ,degree, output, Atype, ef, pa, aa, alpha):
    retY = PutACoeffArray(psd, pixel, eiArray, sampleFileName, Atype, ef, pa, aa, alpha)
    retP = ExecPolyFit(eiArray, retY, degree)
    WriteAbsorptionCoefficientFile(output,psd,pixel,retP,"a")
#-------------------------------------------------------------------------------------------------
def CalcACoeffArrayMulth(psd, pixel, eiArray, sampleFileName ,degree, output, Atype, ef, pa, aa, alpha):

    def func(pix,queue):
        retY = PutACoeffArray(psd, pix, eiArray, sampleFileName, Atype, ef, pa, aa, alpha)
        queue.put(retY)

    MULTH=12
    pixs=[]
    div=int(math.ceil(float(len(pixel))/MULTH))
    for i in range(MULTH-1):
        pixs.append( pixel[div*(i):div*(i+1)])
    pixs.append( pixel[div*(i+1):])

    q = [multiprocessing.Queue() for i in range(MULTH)]
    p = [multiprocessing.Process(target=func, args=(pixs[i],q[i])) for i in range(MULTH)]

    for i in range(MULTH):p[i].start()

    for i in range(MULTH):p[i].join()

    retY=[]
    for i in range(MULTH):
        retY += q[i].get()[0]
    retY = [retY]

    retP = ExecPolyFit(eiArray, retY, degree)
    WriteAbsorptionCoefficientFile(output,psd,pixel,retP,"a")
#-------------------------------------------------------------------------------------------------
def CalcACoeffOfElasticArray(psd, pixel, sampleFileName, output, Atype, ef, pa, aa):
    retY = PutACoeffOfElasticArray(psd, pixel, sampleFileName, "ASS", ef, pa, aa)
    WriteAbsorptionCoefficientFileOfElastic(output,retY,"a")
#-------------------------------------------------------------------------------------------------
def CalcACoeffOfElasticArrayMulth(psd, pixel, sampleFileName, output, Atype, ef, pa, aa):
    def func(pix,queue):
        retY = PutACoeffOfElasticArray(psd, pix, sampleFileName, Atype, ef, pa, aa)
        queue.put(retY)

    MULTH=12
    pixs=[]
    div=int(math.ceil(float(len(pixel))/MULTH))
    for i in range(MULTH-1):
        pixs.append( pixel[div*(i):div*(i+1)])
    pixs.append( pixel[div*(i+1):])

    q = [multiprocessing.Queue() for i in range(MULTH)]
    p = [multiprocessing.Process(target=func, args=(pixs[i],q[i])) for i in range(MULTH)]

    for i in range(MULTH):p[i].start()

    for i in range(MULTH):p[i].join()

    retY=[]
    for i in range(MULTH):
        retY += q[i].get()[0]

    WriteAbsorptionCoefficientFileOfElastic(output,retY,"a")
#-------------------------------------------------------------------------------------------------
def PutACoeffArray(psd, pixel, eiArray, sampleFileName, Atype, ef, pa, aa, alpha=1):

    flagPsd   = isinstance(psd,(list,tuple))
    flagPixel = isinstance(pixel,(list,tuple))

    t = md.AdvCorrectionAbsorptionDNA(sampleFileName)
    RADIAN2DEGREE = 180.0/numpy.pi

    t.SetSigmaCoefficient(1.0, 1.0, alpha)

    if Atype=="ASS":
        GetA=t.GetASS
    elif Atype=="ACSC":
        GetA=t.GetACSC
    elif Atype=="ACSS":
        GetA=t.GetACSS
    elif Atype=="ACC":
        GetA=t.GetACC
    else:
        return [[]]

    if not flagPsd:
        psd=[psd]
    if not flagPixel:
        pixel=[pixel]
    ret =[]
    for ps in psd:
        retTmp=[]
        for pi in pixel:
            polarAngle   = pa[ps][pi]*RADIAN2DEGREE
            azimuthAngle = aa[ps][pi]*RADIAN2DEGREE
            energyFinal  = ef[ps][pi]
            retY = numpy.array([GetA(polarAngle, azimuthAngle, ei, energyFinal) for ei in eiArray])

            retTmp.append(retY)
        ret.append(retTmp)

    del t
    return ret
#-------------------------------------------------------------------------------------------------
def PutACoeffOfElasticArray(psd, pixel, sampleFileName, Atype, ef, pa, aa):

    flagPsd   = isinstance(psd,(list,tuple))
    flagPixel = isinstance(pixel,(list,tuple))

    t = md.AdvCorrectionAbsorptionDNA(sampleFileName)
    RADIAN2DEGREE = 180.0/numpy.pi
    if Atype=="ASS":
        GetA=t.GetASS
    elif Atype=="ACSC":
        GetA=t.GetACSC
    elif Atype=="ACSS":
        GetA=t.GetACSS
    elif Atype=="ACC":
        GetA=t.GetACC
    else:
        return [[]]

    if not flagPsd:
        psd=[psd]
    if not flagPixel:
        pixel=[pixel]
    ret =[]
    for ps in psd:
        retTmp=[]
        for pi in pixel:
            polarAngle   = pa[ps][pi]*RADIAN2DEGREE
            azimuthAngle = aa[ps][pi]*RADIAN2DEGREE
            energyFinal  = ef[ps][pi]
            retY = GetA(polarAngle, azimuthAngle, energyFinal, energyFinal)
            retTmp.append(retY)
        ret.append(retTmp)

    del t
    return ret

#-------------------------------------------------------------------------------------------------
def ExecPolyFit(xarray,yarray,degree):
    if  isinstance(yarray[0],(list,tuple,numpy.ndarray)):
        ret = [ExecPolyFit(xarray,ys,degree) for ys in yarray]
        return ret
    else:
        ret = FitPolyFunc(xarray,yarray,degree)
        return ret
#-------------------------------------------------------------------------------------------------
def FitPolyFunc(xarray,yarray,degree):

    if degree>0:
        popt = numpy.polyfit(xarray,yarray,degree)
        #if len(popt)=4, y = popt[0]*x^3 + popt[1]*x^2 + popt[2]*x +popt[3]
        yexp = numpy.poly1d(popt)(xarray)
        chisq,p=scipy.stats.chisquare(yarray,yexp)
    else:
        popt=[]
        chisq=0
    return popt
#-------------------------------------------------------------------------------------------------
def WriteAbsorptionCoefficientFile(filename,psd,pixel,param,mode):
    # mode="w", write
    # mode="a", append
    sep = ","

    flagPsd   = isinstance(psd,(list,tuple))
    flagPixel = isinstance(pixel,(list,tuple))
    if not flagPsd:
        psd=[psd]
    if not flagPixel:
        pixel=[pixel]

    f=open(filename,mode)
    for i in range(len(psd)):
        for j in range(len(pixel)):
            text="%d,%d,"%(psd[i],pixel[j])+sep.join(["%.4e"%val for val in param[i][j]])+"\n"
            f.write(text)

    f.close()
#-------------------------------------------------------------------------------------------------
def ReadAbsorptionCoefficientFile(filename):
    f=open(filename)
    sep=","

    d=dict()
    for line in f:
        ll = line.split(sep)

        psd   = int(ll[0])
        if not d.has_key(psd):
            d[psd]=dict()

        pixel = int(ll[1])
        param = [float(p) for p in ll[2:]]
        d[psd][pixel]=param

    return d
#-------------------------------------------------------------------------------------------------
def ExecAbsorptionCorrection(ECM,ds,modes):

    ecaSize = ECM.PutTableSize()
    ecm = mm.ElementContainerMatrix(ECM.PutHeader())
    for i in range(ecaSize):
        if i%22==1:
            print str(int(i/22*10)) +" %"

        ECA = ECM.Put(i)
        eca = mm.ElementContainerArray(ECA.PutHeader())
        ecSize = ECA.PutTableSize()
        for j in range(ecSize):

            EC = ECA.Put(j)
            ec=mm.ElementContainer(EC.PutHeader())

            ecXn = numpy.array(EC.PutXList())
            ecX  = (ecXn[1:]+ecXn[:-1])*0.5
            ecY  = numpy.array(EC.PutYList())

            if _checkListDepth(ds)==3:   # apply single absp. coefficient
                abcn = PutAbsorptionCoefficient(ecX,ds,i,j)
                # if modes==0:
                if modes==1:             # corrected by M. Matsuura 2015.3.3 
                    ecYnew = ecY*abcn    # numpy.array * numpy.array
                else:
                    ecYnew = ecY/abcn    # numpy.array / numpy.array
            elif _checkListDepth(ds)==4: # apply some absp. coefficients
                abcns = [PutAbsorptionCoefficient(ecX,d,i,j) for d in ds]
                for abcn, mode in zip(abcns,modes):
                    if mode==0:
                        ecY = ecY*abcn   # numpy.array * numpy.array
                    else:
                        ecY = ecY/abcn   # numpy.array / numpy.array
                    ecYnew = ecY

            ecXnew = ecXn
            ecEnew = numpy.array(EC.PutEList())
            xkey,ykey,ekey    = [EC.PutXKey(),EC.PutYKey(),EC.PutEKey()]
            xunit,yunit,eunit = [EC.PutUnit(key) for key in [xkey,ykey,ekey]]

            ec.Add(xkey,ecXnew.tolist())
            ec.Add(ykey,ecYnew.tolist())
            ec.Add(ekey,ecEnew.tolist())
            ec.SetKeys(xkey,ykey,ekey)
            ec.SetUnit(xkey,xunit)
            ec.SetUnit(ykey,yunit)
            ec.SetUnit(ekey,eunit)

            errMaskArray   = ecEnew>=0
            ecYVal         = ecYnew[errMaskArray]
            sumVal         = numpy.sum(ecYVal)
            ech=ec.PutHeaderPointer()
            ech.OverWrite("TotalCounts",sumVal)

            eca.Add(ec)
        ecm.Add(eca)
    return ecm
#-------------------------------------------------------------------------------------------------
def SubstractionEmptyCellData(DAT1,DAT2):
    ecaSize = DAT1.PutTableSize()
    ecm = mm.ElementContainerMatrix(DAT1.PutHeader())
    for i in range(ecaSize):
        if i%22==1:
            print str(int(i/22*10)) +" %"

        ECA1 = DAT1.Put(i)
        ECA2 = DAT2.Put(i)
        eca = mm.ElementContainerArray(ECA1.PutHeader())
        ecSize = ECA1.PutTableSize()
        for j in range(ecSize):
            EC1 = ECA1.Put(j)
            EC2 = ECA2.Put(j)
            ec=mm.ElementContainer(EC1.PutHeader())

            ecX = numpy.array(EC1.PutXList())

            ecY1 = numpy.array(EC1.PutYList())
            ecE1 = numpy.array(EC1.PutEList())

            ecY2 = numpy.array(EC2.PutYList())
            ecE2 = numpy.array(EC2.PutEList())

            ecI1 = ecE1 != -1
            ecI2 = ecE2 != -1
            ecI  = numpy.logical_and(ecI1,ecI2)

            ecXnew = ecX
            ecYnew = (ecY1 - ecY2)*ecI
            ecEnew = numpy.sqrt(ecY1*ecY1 + ecY2*ecY2)*ecI

            xkey,ykey,ekey    = [EC1.PutXKey(),EC1.PutYKey(),EC1.PutEKey()]
            xunit,yunit,eunit = [EC1.PutUnit(key) for key in [xkey,ykey,ekey]]
            ec.Add(xkey,ecXnew.tolist())
            ec.Add(ykey,ecYnew.tolist())
            ec.Add(ekey,ecEnew.tolist())
            ec.SetKeys(xkey,ykey,ekey)
            ec.SetUnit(xkey,xunit)
            ec.SetUnit(ykey,yunit)
            ec.SetUnit(ekey,eunit)

            errMaskArray   = ecEnew>=0
            ecYVal         = ecYnew[errMaskArray]
            sumVal         = numpy.sum(ecYVal)

            ech=ec.PutHeaderPointer()
            ech.OverWrite("TotalCounts",sumVal)
            eca.Add(ec)
        ecm.Add(eca)

    return ecm
#-------------------------------------------------------------------------------------------------
def DoAbsorptionCorrection(DAT,filename,mode=1):

    filenameFlag = isinstance(filename, (list,tuple))
    if filenameFlag:
        d = [ReadAbsorptionCoefficientFile(name) for name in filename]
    else:
        d = ReadAbsorptionCoefficientFile(filename)

    ECM = ExecAbsorptionCorrection(DAT,d,mode)
    return ECM
#-------------------------------------------------------------------------------------------------
def DoAbsorptionCorrectionWithoutCellData(ECM,filename):
    print "exec Absorption Correction"
    #print "exec *A           : Start"
    print "exec /A           : Start" # Corrected by M. Matsuura 2015.3.3
    ECM_I_S = DoAbsorptionCorrection(ECM, [filename], [0])
    #print "exec *A           : End"
    print "exec /A           : End" # Corrected by M. Matsuura 2015.3.3    
    return ECM_I_S
#-------------------------------------------------------------------------------------------------
def DoAbsorptionCorrectionWithCellData(ECM_I_SC_SC,ECM_I_C_C,AcscFname,AccFname,AsscFname):
    print "exec Absorption Correction"
    print "exec *Acsc / Acc  : Start"
    ECM_I_C_SC  = DoAbsorptionCorrection(ECM_I_C_C,  [AcscFname,AccFname], [1,0])
    print "exec *Acsc / Acc  : End"
    print "exec Iscsc - Icsc : Start"
    ECM_I_S_SC  = SubstractionEmptyCellData(ECM_I_SC_SC, ECM_I_C_SC)
    print "exec Iscsc - Icsc : End"
    # print "exec *Assc        : Start"
    print "exec /Assc        : Start" # Corrected by M. Matsuura 2015.3.3
    ECM_I_S     = DoAbsorptionCorrection(ECM_I_S_SC, [AsscFname], [0])
    # print "exec *Assc        : End"
    print "exec /Assc        : End"  # Corrected by M. Matsuura 2015.3.3
    return ECM_I_S
#-------------------------------------------------------------------------------------------------
def WriteAbsorptionCoefficientFileOfElastic(filename,param,mode):
    sep = ","
    f=open(filename,mode)

    text = sep.join(["%.4e"%val for val in param])+"\n"
    f.write(text)
    f.close()
#-------------------------------------------------------------------------------------------------
def ReadAbsorptionCoefficientFileOfElastic(filename):
    f=open(filename)
    sep=","

    v=[]
    for line in f:
        ll = line.split(sep)
        p=[float(val) for val in ll]

        v.append(p)
    return v
#-------------------------------------------------------------------------------------------------
def ExecAbsorptionCorrectionOfElastic(ECM,d,mode=0):
    ecaSize = ECM.PutTableSize()
    ecm = mm.ElementContainerMatrix(ECM.PutHeader())

    for i in range(ecaSize):
        if i%22==1:
            print str(int(i/22*10)) +" %"

        ECA = ECM.Put(i)
        eca = mm.ElementContainerArray(ECA.PutHeader())
        ecSize = ECA.PutTableSize()
        for j in range(ecSize):
            EC = ECA.Put(j)
            if mode==1:
                ec = EC.Mul(d[i][j])
            else:
                r=d[i][j]
                if r!=0:
                    ec = EC.Mul(1.0/d[i][j])
                else:
                    ec = EC.Mul(0.0)
            ech    = ec.PutHeaderPointer()

            ecY    = numpy.array(ec.PutYList())
            ecE    = numpy.array(ec.PutEList())
            errMaskArray = ecE>=0
            ecY          = ecY[errMaskArray]
            sumVal = numpy.sum(ecY)

            ech.OverWrite("TotalCounts",sumVal)

            eca.Add(ec)
        ecm.Add(eca)
    return ecm
#-------------------------------------------------------------------------------------------------
def DoAbsorptionCorrectionOfElastic(DAT,filename):
    print "exec Absorption Correction"
    # print "exec *A(E=E_ela)  : Start"
    print "exec /A(E=E_ela)  : Start" # Corrected by M. Matsuura 2015.3.3
    d   = ReadAbsorptionCoefficientFileOfElastic(filename)
    ECM = ExecAbsorptionCorrectionOfElastic(DAT,d)
    # print "exec *A(E=E_ela)  : End"
    print "exec /A(E=E_ela)  : End"  # Corrected by M. Matsuura 2015.3.3
    return ECM
#-------------------------------------------------------------------------------------------------
def PutAbsorptionCoefficient(xarray,paramDict,psd,pixel):
    if paramDict.has_key(psd):
        if paramDict[psd].has_key(pixel):
            return PolyFunction(xarray,paramDict[psd][pixel])
    print "PSD:%d,PIXEL:%d data is not found"%(psd,pixel)
    return numpy.ones(len(xarray))
#-------------------------------------------------------------------------------------------------
def PolyFunction(xarray,param):
    return numpy.poly1d(param)(xarray)
#-------------------------------------------------------------------------------------------------
def CalcAbsorptionCoeff(mode,eiArray, ef, aa, pa, sampleFileName):
    t  = md.AdvCorrectionAbsorptionDNA(sampleFileName)
    if mode=="ACC":
        av = [ t.GetACC(pa,aa,ei,ef) for ei in eiArray ]
    elif mode=="ACSS":
        av = [ t.GetACSS(pa,aa,ei,ef) for ei in eiArray ]
    elif mode=="ACSC":
        av = [ t.GetACSC(pa,aa,ei,ef) for ei in eiArray ]
    else:
        av = [ t.GetASS(pa,aa,ei,ef) for ei in eiArray ]
    ave= [ 0.0 for ei in eiArray ]

    EC = mm.ElementContainer()

    eid = (eiArray[1] - eiArray[0])
    eiArray = [val - eid * 0.5 for val in eiArray]
    eiArray.append(eiArray[-1]+eid)

    EC.Add("x",eiArray)
    EC.Add("y",av)
    EC.Add("e",ave)
    EC.SetKeys("x","y","e")

    return EC
#-------------------------------------------------------------------------------------------------
def ExecPolyFitEC(ec,degree):

    ecX0 = ec.PutXList()
    ecX  = (( numpy.array(ecX0)[1:] + numpy.array(ecX0)[:-1])*0.5).tolist()
    ecY  = ec.PutYList()

    param  = ExecPolyFit(ecX,ecY,degree)


    ecXnew = numpy.arange(ecX[0],ecX[-1],(ecX[-1]-ecX[0])/100.0)

    ecYnew = PolyFunction(ecXnew,param).tolist()
    ecXnew -= (ecXnew[1] - ecXnew[0])*0.5
    ecXnew = numpy.hstack([ecXnew,numpy.array([ecXnew[-1]+(ecXnew[1] - ecXnew[0])])]).tolist()

    ecEnew = [0]*len(ecYnew)

    ecOut= mm.ElementContainer()

    ecOut.Add("x",ecXnew)
    ecOut.Add("y",ecYnew)
    ecOut.Add("e",ecEnew)

    ecOut.SetKeys("x","y","e")

    return ecOut
#-------------------------------------------------------------------------------------------------
def PutScatteringCrossSectionFactor():
    return 1
#-------------------------------------------------------------------------------------------------
def _checkListDepth(val,i=0):
    if isinstance(val,(list,tuple,dict)) > 0:
        i = _checkListDepth(val[0],i) + 1
    else:
        pass
    return i
#-------------------------------------------------------------------------------------------------



# -*- coding: utf-8 -*-
"""

"""
import time
import uGao.MPlot as mp


# 予約語の定義
ret = None
mp1 = None

x = [0, 3, 5, 6, 9, 11, 12, 13, 14, 16, 18, 21, 22, 23, 24, 38, 40]
y = [0, -0.1, 1.1, 5, 0, 0, 100, 250, 180.3, 90, 0, 0, 2, 14, 0, 0]
er = [1, 1, 1, 2, 1, 1, 6, 7.1, 5, 4, 2, 0, 2, 3, 2, 1]

header = {'RunNo': 'AMT00001',
          'Label': '5-12(24)',
          'Counts': "512",
          '2Theater': "0.125",
          'L2': "65.3215",
          'Fai': "0.879"
          }

xunit = "micro-sec"
yunit = "neutrons"

ec = (x, y, er, header, xunit, yunit)

matrix = None

###################################################


def MPlot(ematrix=ret, psd=0, pixel=64):
    """
    Create 1D Plotter and Draw EC
    @param  ematrix  ElementContainerMatrix
    @param  psd      PSD No.
    @param  pixel    Pixel No.
    @retval mp1    Instance of Plotter
    """
    # 親プロセス終了時に、プロッタを終了させない
    return mp.MPlot(ematrix(psd, pixel), False)

###################################################


def AddToMPlot(mp1=mp1, ematrix=ret, psd=0, pixel=64):
    """
    Add EC to 1D Plotter
    @param  mp1       Instance of 1D Plotter
    @param  ematrix  ElementContainerMatrix
    @param  psd      PSD No.
    @param  pixel    Pixel No.
    @retval None
    """
    mp1.AddData(ec)

###################################################


def NewPlot(mp1=mp1, ematrix=ret, psd=0, pixel=64):
    """
    Add EC to 1D Plotter
    @param  mp1       Instance of 1D Plotter
    @param  ematrix  ElementContainerMatrix
    @param  psd      PSD No.
    @param  pixel    Pixel No.
    @retval None
    """
    # 親プロセス終了時に、プロッタを終了させない
    mp1.NewPlot(ec, False)

###################################################


def ChangePlot(mp1=mp1, pno=1):
    """
    Add EC to 1D Plotter
    @param  mp1        Instance of 1D Plotter
    @param  pno  Plotter No. (1 to max. Plotter No.)
    @retval None
    """
    mp1.ChangePlot(pno)


# 関数名登録ディクショナリ
_functions = {"MPlot": MPlot,
              "AddToMPlot": AddToMPlot,
              "NewPlot": NewPlot,
              "ChangePlot": ChangePlot}



from __future__ import print_function

import Manyo
import os


"""
Base Functions for SAS
File.py
- Save1dDataToText
- Save1dDataToSrlz
- ManyoDeserializationEC
- ManyoDeserializationECM
- ManyoSerialization
- SaveQzToText
- SaveIqToText
- SaveRPMTToText
- SaveQSpaceToText
"""
#########################################
def Save1dDataToText(dat, dirpath = "/home/mlfdev/users", filename="SAS104000.dat", prec=6):
    """
    Save 1d data to text file(s).
    @param dat       (ElementContainer or list of I(q)) 1d data
    @param dirpath   (string)           directory where files are saved
    @param filename  (string)           filename
    @param prec      (int)              precision
    """
    if not os.path.exists(dirpath):
        raise UserWarning("%s dose not exist!!"%dirpath)
    if isinstance(dat, list):
        for ec in dat:
            xKey = ec.PutXKey()
            try:
                ec.SetKeys("Q", "Intensity", "Error")
            except:
                raise UserWarning("Data is not I(q).")
            bank_name = ec.PutHeader().PutString("BANKNAME")
            fn_split = os.path.splitext(filename)
            new_filename = "%s-%s%s"%(fn_split[0], bank_name, fn_split[1])
            print("Save %s to %s"%(bank_name, os.path.join(dirpath, new_filename)))
            ec.SaveTextFile(os.path.join(dirpath, new_filename), prec)
            ec.SetKeys(xKey, "Intensity", "Error")
    else:
        try:
            dat.SaveTextFile(os.path.join(dirpath, filename), prec)
        except:
            raise UserWarning("Unexpected data format")

#########################################
def Save1dDataToSrlz(dat, dirpath = "/home/mlfdev/users", filename="SAS104000.srlz"):
    """
    Save 1d data to text file(s).
    @param dat       (ElementContainer or list of I(q)) 1d data
    @param dirpath   (string)           directory where files are saved
    @param filename  (string)           filename
    """
    if not os.path.exists(dirpath):
        raise UserWarning("%s dose not exist!!"%dirpath)
    if isinstance(dat, list):
        for ec in dat:
            xKey = ec.PutXKey()
            try:
                ec.SetKeys("Q", "Intensity", "Error")
            except:
                raise UserWarning("Data is not I(q).")
            bank_name = ec.PutHeader().PutString("BANKNAME")
            fn_split = os.path.splitext(filename)
            new_filename = "%s-%s%s"%(fn_split[0], bank_name, fn_split[1])
            print("Save %s to %s"%(bank_name, os.path.join(dirpath, new_filename)))
            write = Manyo.WriteSerializationFileBinary(os.path.join(dirpath, new_filename))
            write.Save(ec)
            del write
            ec.SetKeys(xKey, "Intensity", "Error")
    else:
        try:
            write = Manyo.WriteSerializationFileBinary(os.path.join(dirpath, filename))
            write.Save(dat)
            del write
        except:
            raise UserWarning("Unexpected data format")
#########################################
def SaveQzToText(dat, dirpath = "/home/mlfdev/users", filename="SAS104000_Qz.dat"):
    """
    Save Qz to a text file.
    @param dat          Qz ElementContainerArray
    @param dirpath      directory where files are saved
    @param filename     filename
    """
    if not isinstance( dat, Manyo.ElementContainerArray ):
        raise UserWarning("dat is not ElementContainerArray")
    fn = open(os.path.join(dirpath, filename), "w")
    qxs = dat.PutHeaderPointer().PutDoubleVector("Qx")
    num_of_ec = dat.PutSize()
    ##[inamura 170118]-->
    fn.write("##Qx, Qy, Intensity, Error\n")
    qxsize = qxs.size()
    qy = dat(0).ReduceColumnList("Qy")
    qysize = len(qy)
    fn.write("## XRANGE=%f,%f,%f\n"%( (qxs[0]+qxs[1])/2.0, (qxs[qxsize-2]+qxs[qxsize-1])/2.0, qxs[1]-qxs[0] ) )
    fn.write("## YRANGE=%f,%f,%f\n"%( qy[0],qy[qysize-1],qy[1]-qy[0] ) )
    fn.write("## XLABEL=Qx [1/Ang.]\n")
    fn.write("## YLABEL=Qy [1/Ang.]\n")

    out_ymatrix=[]
    out_ematrix=[]
    for i in range(num_of_ec):
        ys = dat(i).PutYList()
        es = dat(i).PutEList()
        out_ymatrix.append( ys )
        out_ematrix.append( es )

    for i in range( num_of_ec ):
        for j in range( qysize ):
            fn.write( "%f, %f, %e, %e\n"%( (qxs[j]+qxs[j+1])/2.0, qy[i], out_ymatrix[j][i], out_ematrix[j][i] ) )

    fn.close()
    """
    for i in range(num_of_ec):
        xs = dat(i).ReduceColumnList("Qy")
        ys = dat(i).PutYList()
        es = dat(i).PutEList()
        for j in range(len(ys)):
            fn.write("%f, %f, %e, %e\n"%( (qxs[i]+qxs[i+1])*0.5, xs[j], ys[j], es[j]))
    """
    ##<--[inamura 170118]
#########################################
def SaveIqToText(dat, dirpath = "/home/mlfdev/users", filename="SAS104000_Iq.dat", prec=6):
    """
    Save I(q) to a text file.
    @param dat         I(q) data which is list of ElementContainers
    @param dirpath      directory where files are saved
    @param filename     filename
    @param prec         precision
    """
    xKey = dat[0].PutXKey()
    try:
        dat[0].SetKeys("Q", "Intensity", "Error")
    except:
        raise UserWarning("Data is not I(q).")
    if not os.path.exists(dirpath):
        raise UserWarning("%s dose not exist!!"%dirpath)
    dat[0].SaveTextFile(os.path.join(dirpath, filename), prec)
    dat[0].SetKeys(xKey, "Intensity", "Error")

#########################################
def ManyoDeserializationEC(dirpath = ".", filename=""):
    """
    Deserialize an EC file by Manyo-Lib.
    @param dirpath              directory where files are saved
    @param filename             filename
    @retval EC                 ElementContainer
    """
    filepath = os.path.join(dirpath, filename)
    if not os.path.exists(filepath):
        raise UserWarning("%s dose not exist!!"%filepath)
    ec = Manyo.ElementContainer()
    read = Manyo.ReadSerializationFileBinary(filepath)
    read.Load(ec)
    del read
    return ec

#########################################
def ManyoDeserializationECM(dirpath = ".", filename=""):
    """
    Deserialize an ECM file by Manyo-Lib.
    @param dirpath              directory where files are saved
    @param filename             filename
    @retval DAT                 ElementContainerMatrix
    """
    filepath = os.path.join(dirpath, filename)
    if not os.path.exists(filepath):
        raise UserWarning("%s dose not exist!!"%filepath)
    ecm = Manyo.ElementContainerMatrix()
    read = Manyo.ReadSerializationFileBinary(filepath)
    read.Load(ecm)
    del read
    return ecm

#########################################
def ManyoSerialization(dat, dirpath = "/home/mlfdev/users", filename="SAS104000.srlz"):
    """
    Serialize data by Manyo-Lib.
    @param dat                  data container
    @param dirpath              directory where files are saved
    @param filepath             filename
    """
    if not os.path.exists(dirpath):
        raise UserWarning("%s dose not exist!!"%dirpath)
    filepath = os.path.join(dirpath, filename)
    write = Manyo.WriteSerializationFileBinary(filepath)
    write.Save(dat)
    del write

#########################################
def SaveRPMTToText(dat, dirpath="/home/mlfdev/users", filename="SAS000001_RPMT.txt", lambdaRange="0.7 7.5"):
    """
    Save RPMT data to text
    @param dat             ElementContainerMatrix
    @param dirpath         directory where files are saved
    @param lambdaRange
    @param filepath        file name
    """
    import math
    if not os.path.exists(dirpath):
        raise UserWarning("%s dose not exist!!"%dirpath)
    filepath = os.path.join(dirpath, filename)

    lambdaRange_list=lambdaRange.strip().split(" ")
    if len(lambdaRange_list)!=2:
        raise UserWarning("Lambda Range is invalid")
    lamb_min = float(lambdaRange_list[0])
    lamb_max = float(lambdaRange_list[1])

    existed_xpos_list = []
    existed_ypos_list = []
    for i in range(dat.PutSize()):
        if dat(i).PutHeaderPointer().PutInt4("MASKED")==0:
            xpos = dat(i).PutHeaderPointer().PutInt4("XPOS")
            existed_xpos_list.append( xpos )
            if len(existed_ypos_list)==0:
                for j in range(dat(i).PutSize()):
                    ypos = dat(i,j).PutHeaderPointer().PutInt4("YPOS")
                    existed_ypos_list.append( ypos )

    existed_xpos_list.sort()
    existed_ypos_list.sort()

    fo = open( filepath, "w" )
    if fo==None:
        raise UserWarning("Failed to open file = "+filepath)

    SIH = Manyo.SearchInHeader()
    SIH.SetTarget(dat)

    for xpos in existed_xpos_list:
        SIH.SearchArray( "XPOS", xpos )
        iv = SIH.PutResultIndex()
        if iv.size()!=0:
            eca=dat(iv[0])
            int_list = []
            err_list = []
            for i in range( len(existed_ypos_list) ):
                int_list.append(0.0)
                err_list.append(0.0)

            for i in range(eca.PutSize()):
                ypos = eca(i).PutHeaderPointer().PutInt4("YPOS")
                ii = eca(i).PutYList()
                ee = eca(i).PutEList()
                LL = eca(i).Put('Lamb')
                for j in range( LL.size() ):
                    if LL[j]>=lamb_min and LL[j]<lamb_max:
                        int_list[ypos] = int_list[ypos]+ii[j]
                        err_list[ypos] = err_list[ypos]+( ee[j]*ee[j] )

            for ypos in range( len(existed_ypos_list) ):
                conts = "%d, %d, %f, %f\n"%(xpos,ypos,int_list[ypos],math.sqrt( err_list[ypos] ))
                fo.write( conts )

    fo.close()

#########################################
def SaveQSpaceToText(dat, dirpath="/home/mlfdev/users", filename="SAS000001_QSpace.txt", QxRange="1.0 1.2", QyRange="-0.1 0.1", QzRange="-0.4 -0.2", cutoff=0.01):
    """
    Save Q space data to text
    @param dat      (ElementContainerMatrix) Q space ECM
    @param dirpath  (string) Directory where files are saved
    @param filename (string) File name
    @param QxRange  (string) Minimum and maximum qx separated with space
    @param QyRange  (string) Minimum and maximum qy separated with space
    @param QzRange  (string) Minimum and maximum qz separated with space
    @param cutoff   (float) Cutoff value for intensity. If it is negative, it is ignored.
    """
    if not os.path.exists(dirpath):
        raise UserWarning("{} dose not exist!!".format(dirpath))
    filepath = os.path.join(dirpath, filename)

    qx_min, qx_max = list(map(float, QxRange.strip().split(" ")))
    qy_min, qy_max = list(map(float, QyRange.strip().split(" ")))
    qz_min, qz_max = list(map(float, QzRange.strip().split(" ")))

    ecm_size = dat.PutSize()
    eca_size = dat(0).PutSize()
    with open(filepath, "wb") as fp:
        for i in range(ecm_size):
            if (dat(i).PutHeaderPointer().PutInt4("MASKED"))==1:
                continue
            for j in range(dat(i).PutSize()):
                if (dat(i, j).PutHeaderPointer().PutInt4("MASKED"))==1:
                    continue
                qx_l = dat(i, j).ReduceColumnList("Qx")
                qy_l = dat(i, j).ReduceColumnList("Qy")
                qz_l = dat(i, j).ReduceColumnList("Qz")
                ys = dat(i, j).PutYList()
                es = dat(i, j).PutEList()
                for k in range(len(qx_l)):
                    if cutoff < 0.0:
                        if ( qx_min <= qx_l[k] <= qx_max ) and ( qy_min <= qy_l[k] <= qy_max ) and ( qz_min <= qz_l[k] <= qz_max ):
                            fp.write("{0:g}, {1:g}, {2:g}, {3:g}, {4:g}\n".format(qx_l[k], qy_l[k], qz_l[k], ys[k], es[k]))
                    else:
                        if (ys[k] >= cutoff) and ( qx_min <= qx_l[k] <= qx_max ) and ( qy_min <= qy_l[k] <= qy_max ) and ( qz_min <= qz_l[k] <= qz_max ):
                            fp.write("{0:g}, {1:g}, {2:g}, {3:g}, {4:g}\n".format(qx_l[k], qy_l[k], qz_l[k], ys[k], es[k]))
                del qx_l, qy_l, qz_l, ys
    del dat



from __future__ import print_function

import sys
import Manyo as mm
import Manyo.Utsusemi as mu
import Manyo.Emaki as me

if len(sys.argv) !=4:
    print("Usage: %s run_number wiring_info detector_info"%sys.argv[0])
    print("Note:")
    print("")
    sys.exit()

RunNumber = int(sys.argv[1])
Wfile = sys.argv[2]
Dfile = sys.argv[3]
InstCode = 'ENG'
DataPath = '/data'
output_path = "%s%06d.srlz"%(InstCode, RunNumber)

evt = me.EmakiEventDataConverterReadout1d()
evt.LoadParamFiles( Wfile, Dfile )
evt.SetHistAllocation()
ecm = mm.ElementContainerMatrix()
evt.SetElementContainerMatrix( ecm, RunNumber, DataPath )

boostWrite = mm.WriteSerializationFileBinary(output_path)
boostWrite.Save(ecm)
print("[o] Histograms were serialized to %s."%output_path)




# -*- coding: utf-8 -*-
#
# $Date: 2012-08-20 17:49:07 +0900 (月, 20 8 2012) $
# $Rev: 2 $
#
import wx
from wx import xrc

import os

class FileNotFound(Exception):
    def __init__(self, filename):
        self.filename = filename
        return

    def __str__(self):
        return repr(self.filename)

class ControlNotFound(Exception):
    def __init__(self, name):
        self.name = name
        return

    def __str__(self):
        return 'name: ' + repr(self.name)

class XrcBase(object):
    __resources = dict()

    @staticmethod
    def getResource(filename):
        fullpath = os.path.join(os.path.abspath(os.path.dirname(__file__)), '../../')
        fullpath = os.path.join(fullpath, filename)

        if not os.path.exists(fullpath):
            raise FileNotFound(fullpath)

        if filename in XrcBase.__resources:
            resource = XrcBase.__resources[filename]
            pass
        else:
            resource = xrc.EmptyXmlResource()
            resource.Load(filename)
            XrcBase.__resources[filename] = resource
            pass

        return resource

    def getControl(self, name):
        control = xrc.XRCCTRL(self, name)

        if control == None:
            raise ControlNotFound(name)
        return control

    def setNumberInput(self, name):
        obj=self.getControl(name)

        if "SetValue" in dir(obj):
            obj.Bind(wx.EVT_TEXT,self.checkNumberInput)

    def checkNumberInput(self,event):       # テキストコントロールに数字だけ入力モードにする。
        obj=event.GetEventObject()
        text=obj.GetValue()
        if len(text)!=0: 
            condition  = text[-1].isdigit() # isdigit
            condition += text[-1]=="."      # とりあえず可 OR
            condition += text[-1]=="+"      #           OR
            condition += text[-1]=="-"      #           OR
            condition += text[-1]=="e"      #           OR
            condition += text[-1]=="E"      #           OR
            #AND
            condition *=text.count(".")<2   # 2個以上は不可
            condition *=text.count("e")<2
            condition *=text.count("E")<2
            #AND
            if text[-1]=="+":
                if len(text)==1:            # 最初は可
                    condition *=True
                else:
                    if text[-2]=="e" or text[-2]=="E": # Eの次でも可
                        condition *=True
                    else:
                        condition *=False
            #AND
            if text[-1]=="-":
                if len(text)==1:            # 最初は可
                    condition *=True
                else:
                    if text[-2]=="e" or text[-2]=="E": # Eの次でも可
                        condition *=True
                    else:
                        condition *=False

            if condition > 0:
                pass
            else:
                if len(text)!=0:
                    obj.SetValue(text[:-1])


class FrameBase(XrcBase, wx.Frame):
    def __init__(self, filename, frameName):
        resource = XrcBase.getResource(filename)

        pre = wx.PreFrame()
        resource.LoadOnFrame(pre, None, frameName)
        self.PostCreate(pre)
        return

class DialogBase(XrcBase, wx.Dialog):
    def __init__(self, parent, filename, dialogName):
        resource = XrcBase.getResource(filename)

        pre = wx.PreDialog()
        resource.LoadOnDialog(pre, parent, dialogName)
        self.PostCreate(pre)
        self.CenterOnParent()
        return

class PanelBase(XrcBase, wx.Panel):
    def __init__(self, parent, filename, panelName):
        resource = XrcBase.getResource(filename)

        pre = wx.PrePanel()
        resource.LoadOnPanel(pre, parent, panelName)
        self.PostCreate(pre)
        return



# Distributed parallel data analysis environment with python script
# has been prepared in Manyo Library with Boost C++ library.

# This environment should be executed with Boost C++ ver. 1.46.1.
# To solve this problem, we will prepare OpenMPI wrapper in future.

# The binary file of remote server has been built and installed
# into the targeted directory.
# The file name of source code of the remote server is
# "SerialPythonServerMain.cpp".


import Manyo
import Manyo.SerialPySrv as MS

# preparing data container which will be send to the remote server
# and analyze on the remote.
e1 = Manyo.ElementContainer()
vec = Manyo.MakeDoubleVector( 100 )
for i in range( 100 ):
    e1.Add( "x"+str(i), vec )

# Making the class object for parallel processing
cli = MS.SerialPythonServerClient( "localhost", "8888" )

# This object, vector<string>, will be executed on the remote server.
# The data object of "EX" in the script is prepared automatically.
PyCom = Manyo.MakeStringVector()
PyCom.push_back( "e = Manyo.ElementContainer()" )
PyCom.push_back( "EX.Receive(e)" )
PyCom.push_back( "EX.Transmit(e)" )
PyCom.push_back( "del( e )" )


# The script is transmitted to the remote server with
# the value of time-out of 3000 milli-seconds.
# The return value of this method is True or False.
Flag = cli.Start( PyCom, 3000 )


# If the return value is True, you can communicate with the remote server.
if Flag == True:
    Nex = cli.refSNE()
    Nex.Transmit( e1 )
    er = Manyo.ElementContainer()
    Nex.Receive( er )




#!/usr/bin/python

import Manyo as m
import Peak as p

peak0 = p.Peak(p.SHOLDER_I, -2.5,  50000.0, 4.5, -7.0, -2.5)
peak1 = p.Peak(p.PEAK,       0.0, 100000.0, 2.5, -2.5,  2.5)
peak2 = p.Peak(p.SHOLDER_D,  2.5,  50000.0, 4.5,  2.5,  7.0)
p.Peak.title()
peak0.dump()
peak1.dump()
peak2.dump()

peak4 = p.Peak(peak1)
peak5 = peak2
peak4.dump()
peak5.dump()

v=peak0.toVector()
for i in range(p.Peak.shortSize):
    print "v[", i, "]=", v[i]
print ""

v=peak0.toFullVector()
for i in range(p.Peak.fullSize):
    print "v[", i, "]=", v[i]
print ""

print p.PEAK.value, p.PEAK.symbol





import logging
import sys
import time
import select
import optparse
import OptionSelector
import random

import Manyo
import MultiDataPeakFit as Adv

#import PlotNorm
#import PeakFilter
#import FuncModelGen
#import LowerAndUpperBoundGen

def displace(param):
    #if not isinstance(param, DoubleVector):
    #if not isinstance(param, VectorDouble):
    if not isinstance(param, Manyo.VecDouble):
        raise TypeError("randamize: unsupported type")

    for i in range(len(param)):
        param[i]=param[i]*(0.5 + random.random())
        print "displace: i=", i, "value=", param[i]

########        command line operations        ########
parser=optparse.OptionParser()
parser.add_option("-i", "--input", action="store", type="string", dest="inputFile", help="input file name", metavar="FILE")
options, args = parser.parse_args()
print "options=", options
print "args   =", args
#if len(args) == 0:
#    parser.error("no argument")
if not options.inputFile:
    parser.error("no input file specified.")

########        read original data        ########
io=Manyo.NeXusFileIO()
src = io.ReadElementContainerArray(options.inputFile)
print "container size: ", src.PutSize()

#exprList=Manyo.VectorTool.MakeVectorString()
exprList=[]
for i in range(src.PutSize()):
    ec = src.Put(i)
    exprList.append(ec.PutHeader().PutString("source functions"))
    print exprList[i]

cppToPython=Manyo.CppToPython()
vecString = cppToPython.ListToStringVector(exprList)
param = src.Put(0).PutHeader().PutDoubleVector("source parameters")

op = Adv.MultiDataPeakFit()
op.setSource(src)
op.setMethod(Adv.MULTIDATA_LEVMAR)
op.setDefaultParam()
displace(param)
print "param", param

op.setParam(Adv.LevmarConsts.CONSTRAIN,          Adv.LevmarConsts.NO_CONSTRAIN)
op.setParam(Adv.LevmarConsts.USE_NUMERICAL_DIFF, Adv.LevmarConsts.DEFAULT_USE_NUMERICAL_DIFF)
op.setParam(Adv.LevmarConsts.DIFF_METHOD,        Adv.LevmarConsts.DEFAULT_DIFF_METHOD)
op.setParam(Adv.LevmarConsts.USE_DATA_WEIGHTS,   Adv.LevmarConsts.DEFAULT_USE_DATA_WEIGHTS)

op.setParam(Adv.LevmarConsts.FUNCTIONS,        vecString)
op.setParam(Adv.LevmarConsts.PARAMETER_VALUES, param)
op.getParam().dump()

#plotNorm=PlotNorm()

if op.checkParam():
    #op.execute()
    op.fit()
    while op.isFitting():
        p=op.getLatestConvergenceStat()
        #plotNorm.add(p)
        #plotNorm.plot()
        waitTime=p.getInt4(Adv.LevmarConsts.ITERATION_TIME)/1000000.0
        #print waitTime
        tmpList=select.select([sys.stdin], [], [], waitTime)
        #print tmpList
        if not tmpList==([], [], []):
            c=raw_input()
            if c=="q":
                drv.stopFit()
                break
    while op.isFitting(): # sleep while fitting
        pass
    #plotNorm.addNorm(drv.getLatestConvergenceStat())
    #plotNorm.plot()
    op.eval()

    result=op.getResult()
    result.PutHeader().dump()
    for i in range(src.PutSize()):
        result.Put(i).PutHeader().Dump()
    resultComponents=op.getResultComponents()

    output = Manyo.NeXusFileIO()
    output.Write(result, "fittedArray.data", "tanimori")
